/* Directives */

peakApp
    // page preloader
    .directive('pageLoader', [
        '$timeout',
        function ($timeout) {
            return {
                restrict: 'AE',
                template: '<div class="rect1"></div><div class="rect2"></div><div class="rect3"></div><div class="rect4"></div><div class="rect5"></div>',
                link: function (scope, el, attrs) {
                    el.addClass('pageLoader hide');
                    scope.$on('$stateChangeStart', function (event) {
                        el.toggleClass('hide animate');
                    });
                    scope.$on('$stateChangeSuccess', function (event, toState, toParams, fromState) {
                        event.targetScope.$watch('$viewContentLoaded', function () {
                            $timeout(function () {
                                //el.toggleClass('hide animate')
                                el.addClass('pageLoader hide');// Had to add this hack to kill the extra animation
                            }, 600);
                        })
                    });
                }
            };
        }
    ])

    // show/hide side menu
    .directive('menuToggle', [
        '$rootScope',
        '$cookieStore',
        '$window',
        function ($rootScope, $cookieStore, $window) {
            return {
                restrict: 'E',
                template: '<span class="menu_toggle" ng-click="toggleSidebar()"><span class="icon_menu_toggle" ><i class="arrow_carrot-2left" ng-class="sideNavCollapsed ? \'hide\' : \'\'"></i><i class="arrow_carrot-2right" ng-class="sideNavCollapsed ? \'\' : \'hide\'"></i></span></span>',
                link: function (scope, el, attrs) {
                    var mobileView = 992;
                    $rootScope.getWidth = function () {
                        return window.innerWidth;
                    };
                    $rootScope.$watch($rootScope.getWidth, function (newValue, oldValue) {
                        if (newValue >= mobileView) {
                            if (angular.isDefined($cookieStore.get('sideNavCollapsed'))) {
                                if ($cookieStore.get('sideNavCollapsed') == false) {
                                    $rootScope.sideNavCollapsed = false;
                                }
                                else {
                                    $rootScope.sideNavCollapsed = true;
                                }
                            }
                            else {
                                $rootScope.sideNavCollapsed = false;
                            }
                        }
                        else {
                            $rootScope.sideNavCollapsed = true;
                        }
                    });
                    scope.toggleSidebar = function () {
                        $rootScope.sideNavCollapsed = !$rootScope.sideNavCollapsed;
                        $cookieStore.put('sideNavCollapsed', $rootScope.sideNavCollapsed);
                    };
                    window.onresize = function () {
                        $rootScope.$apply();
                    };

                }
            };
        }
    ])

    .directive('focusOnInput', function ($timeout) {
        return {
            link: function (scope, element, attrs) {
                scope.$watch(attrs.focusOnInput, function (value) {
                    if (value === true) {
                        console.log('Setting Default Focus for Input on Modal');
                        $timeout(function () {
                            element[0].focus();
                            scope[attrs.focusMe] = false;
                        });                    
                    }
                });
            }
        };
    })

    // update datatables fixedHeader position
    .directive('updateFixedHeaders', function ($window) {
        return function (scope, element) {
            var w = angular.element($window);
            scope.getElDimensions = function () {
                return {
                    'w': element.width(),
                    'h': element.height()
                };
            };
            scope.$watch(scope.getElDimensions, function (newValue, oldValue) {
                if (typeof oFH != 'undefined') {
                    oFH._fnUpdateClones(true);
                    oFH._fnUpdatePositions();
                }
            }, true);
            w.bind('resize', function () {
                scope.$apply();
            });
        };
    })

    // ng-repeat after render callback
    .directive('onLastRepeat', function ($timeout) {
        return function (scope, element, attrs) {
            if (scope.$last) {
                $timeout(function () {
                    scope.$emit('onRepeatLast', element, attrs);
                })
            }
        };
    })

    // add width/height properities to Image
    .directive('addImageProp', function () {
        return {
            restrict: 'A',
            link: function (scope, elem, attr) {
                elem.on('load', function () {
                    var w = !scope.isHighDensity() ? $(this).width() : $(this).width() / 2,
                        h = !scope.isHighDensity() ? $(this).height() : $(this).height() / 2;
                    $(this).attr('width', w).attr('height', h);
                });
            }
        };
    })

    // dynamically adjust Digital Inputs display
    .directive('footableDigitalInputs', function () {
        return function (scope, element) {
            var footableTable = element.parents('table');// Get the parent of this element
            
            // If this isn't the $last ng-repeat, then ignore
            if (!scope.$last) {
                return false;
            }
            
            
            // This is the $last ng-repeat, wait and then initialize the footable table
            scope.$evalAsync(function () {
                console.log("Initializing footableDigitalInputs");
                if (!footableTable.hasClass('footable-loaded')) {
                    footableTable.footable({
                        toggleSelector: " > tbody > tr > td > span.footable-toggle"
                    })
                }
                
                console.log("footableDigitalInputs directive: length of scope.DigitalInputs=" + scope.DigitalInputs.length);
                for (i = 0, enabled = 0; i < scope.DigitalInputs.length; i++) {
                    if (scope.DigitalInputs[i].Enabled === "TRUE") {
                        if (scope.DigitalInputs[i].Value == "1") {
                            $('#IdDigitalInput' + i + 'Value').html("ON");
                            $('#IdDigitalInput' + i + 'Value').css('color', 'white');
                            $('#IdDigitalInput' + i + 'Value').css('background-color', 'green');
                            $('#IdDigitalInput' + i + 'Value').css('border-color', 'green');
                        }
                        else {
                            $('#IdDigitalInput' + i + 'Value').html("OFF");
                            $('#IdDigitalInput' + i + 'Value').css('color', 'white');
                            $('#IdDigitalInput' + i + 'Value').css('background-color', 'red');
                            $('#IdDigitalInput' + i + 'Value').css('border-color', 'red');
                        }
                    }
                    else {
                        $('#IdDigitalInput' + i + 'Value').html("--");
                        $('#IdDigitalInput' + i + 'Value').css('color', 'white');
                        $('#IdDigitalInput' + i + 'Value').css('background-color', 'grey');
                        $('#IdDigitalInput' + i + 'Value').css('border-color', 'grey');
                    }
                }
                
                footableTable.trigger('footable_initialized');
                footableTable.trigger('footable_resize');
                footableTable.data('footable').redraw();

            });
        };
    })

    // dynamically adjust Digital Outputs display
    .directive('footableDigitalOutputs', function () {
        return function (scope, element) {
            var footableTable = element.parents('table');// Get the parent of this element
            
            // If this isn't the $last ng-repeat, then ignore
            if (!scope.$last) {
                return false;
            }
            
            
            // This is the $last ng-repeat, wait and then initialize the footable table
            scope.$evalAsync(function () {
                console.log("Initializing footableDigitalOutputs");
                if (!footableTable.hasClass('footable-loaded')) {
                    footableTable.footable({
                        toggleSelector: " > tbody > tr > td > span.footable-toggle"
                    })
                }
                
                console.log("footableDigitalOutputs directive: length of scope.DigitalOutputs=" + scope.DigitalOutputs.length);
                for (i = 0, enabled = 0; i < scope.DigitalOutputs.length; i++) {
                    if (scope.DigitalOutputs[i].Enabled === "TRUE") {
                        if (scope.DigitalOutputs[i].Value == "1") {
                            $('#IdDigitalOutput' + i + 'Value').html("ON");
                            $('#IdDigitalOutput' + i + 'Value').css('color', 'white');
                            $('#IdDigitalOutput' + i + 'Value').css('background-color', 'green');
                            $('#IdDigitalOutput' + i + 'Value').css('border-color', 'green');
                            $('#IdDigitalOutput' + i + 'Set').html("Turn OFF");
                        } else {
                            $('#IdDigitalOutput' + i + 'Value').html("OFF");
                            $('#IdDigitalOutput' + i + 'Value').css('color', 'white');
                            $('#IdDigitalOutput' + i + 'Value').css('background-color', 'red');
                            $('#IdDigitalOutput' + i + 'Value').css('border-color', 'red');
                            $('#IdDigitalOutput' + i + 'Set').html("Turn ON");
                        }
                        // Hard-coded switch back to Primary from grey or unknown
                        $('#IdDigitalOutput' + i + 'Set').css("color", " #fff");
                        $('#IdDigitalOutput' + i + 'Set').css("background-color", '#f0ad4e');
                        $('#IdDigitalOutput' + i + 'Set').css("border-color", '#f0ad4e');
                    }
                    else {
                        $('#IdDigitalOutput' + i + 'Value').html("--");
                        $('#IdDigitalOutput' + i + 'Value').css('color', 'white');
                        $('#IdDigitalOutput' + i + 'Value').css('background-color', 'grey');
                        $('#IdDigitalOutput' + i + 'Value').css('border-color', 'grey');
                        $('#IdDigitalOutput' + i + 'Set').html("--");
                        $('#IdDigitalOutput' + i + 'Set').css("color", "white");
                        $('#IdDigitalOutput' + i + 'Set').css("background-color", 'grey');
                        $('#IdDigitalOutput' + i + 'Set').css("border-color", 'grey');
                    }
                }
                
                footableTable.trigger('footable_initialized');
                footableTable.trigger('footable_resize');
                footableTable.data('footable').redraw();

            });
        };
    })
    
    // dynamically adjust Analog Inputs display
    .directive('footableAnalogInputs', function () {
        return function (scope, element) {
            var footableTable = element.parents('table');// Get the parent of this element
            
            // If this isn't the $last ng-repeat, then ignore
            if (!scope.$last) {
                return false;
            }
            
            
            // This is the $last ng-repeat, wait and then initialize the footable table
            scope.$evalAsync(function () {
                console.log("Initializing footableAnalogInputs");
                if (!footableTable.hasClass('footable-loaded')) {
                    footableTable.footable({
                        toggleSelector: " > tbody > tr > td > span.footable-toggle"
                    })
                }
                
                console.log("footableAnalogInputs directive: length of scope.AnalogInputs=" + scope.AnalogInputs.length);
                for (i = 0, enabled = 0; i < scope.AnalogInputs.length; i++) {
                    if (scope.AnalogInputs[i].Enabled === "TRUE") {
                        scope.AnalogInputs[i].LowThreshold = parseFloat(scope.AnalogInputs[i].LowThreshold).toFixed(2);
                        scope.AnalogInputs[i].HighThreshold = parseFloat(scope.AnalogInputs[i].HighThreshold).toFixed(2);

                        var ANALOG_INPUTS_RAWVALUE_OFFSET = -2.1796;// Default = -2.1796
                        var ANALOG_INPUTS_RAWVALUE_SLOPE = 0.0112;// Default = 0.0112
                        RangedValue = (scope.AnalogInputs[i].TempValue * ANALOG_INPUTS_RAWVALUE_SLOPE + ANALOG_INPUTS_RAWVALUE_OFFSET).toFixed(2);
                        //console.log("Directive.AnalogInput RawValue=" + scope.AnalogInputs[i].RawValue + " TempValue=" + scope.AnalogInputs[i].TempValue +" RangedValue=" + RangedValue);

                        $('#IdAnalogInput' + i + 'Value').html(RangedValue + " Volts");
                        $('#IdAnalogInput' + i + 'Value').css('color', 'white');
                        if (scope.AnalogInputs[i].ThresholdState === 0) {
                            $('#IdAnalogInput' + i + 'Value').css('background-color', 'green');
                            $('#IdAnalogInput' + i + 'Value').css('border-color', 'green');
                        } else {
                            $('#IdAnalogInput' + i + 'Value').css('background-color', 'red');
                            $('#IdAnalogInput' + i + 'Value').css('border-color', 'red');
                        }
                    }
                    else {
                        $('#IdAnalogInput' + i + 'Value').html("--");
                        $('#IdAnalogInput' + i + 'Value').css('color', 'white');
                        $('#IdAnalogInput' + i + 'Value').css('background-color', 'grey');
                        $('#IdAnalogInput' + i + 'Value').css('border-color', 'grey');
                    }
                }

                footableTable.trigger('footable_initialized');
                footableTable.trigger('footable_resize');
                footableTable.data('footable').redraw();

            });
        };
    })

    // dynamically adjust Analog Outputs display    
    .directive('footableAnalogOutputs', function () {
        return function (scope, element) {
            var footableTable = element.parents('table');// Get the parent of this element
            
            // If this isn't the $last ng-repeat, then ignore
            if (!scope.$last) {
                return false;
            }
            
            
            // This is the $last ng-repeat, wait and then initialize the footable table
            scope.$evalAsync(function () {
                console.log("Initializing footableAnalogOutputs");
                if (!footableTable.hasClass('footable-loaded')) {
                    footableTable.footable({
                        toggleSelector: " > tbody > tr > td > span.footable-toggle"
                    })
                }
                
                console.log("footableAnalogOutputs directive: length of scope.AnalogOutputs=" + scope.AnalogOutputs.length);
                for (i = 0, enabled = 0; i < scope.AnalogOutputs.length; i++) {
                    if (scope.AnalogOutputs[i].Enabled === "TRUE") {
                        var ANALOG_OUTPUT_CONVERT_TO_RAW_SLOPE = 200;
                        var ANALOG_OUTPUT_CONVERT_TO_RAW_OFFSET = 0;
                        RangedValue = ((scope.AnalogOutputs[i].RawValue - ANALOG_OUTPUT_CONVERT_TO_RAW_OFFSET) / ANALOG_OUTPUT_CONVERT_TO_RAW_SLOPE).toFixed(2);
                        $('#IdAnalogOutput' + i + 'Value').html(RangedValue + " Volts");
                        $('#IdAnalogOutput' + i + 'Value').css('color', 'white');
                        $('#IdAnalogOutput' + i + 'Value').css('background-color', 'green');
                        $('#IdAnalogOutput' + i + 'Value').css('border-color', 'green');
                        $('#IdAnalogOutput' + i + 'Set').html("Set Value");
                        $('#IdAnalogOutput' + i + 'Set').css("color", "white");
                        $('#IdAnalogOutput' + i + 'Set').css("background-color", '#f0ad4e');
                        $('#IdAnalogOutput' + i + 'Set').css("border-color", '#f0ad4e');
                    }
                    else {

                        $('#IdAnalogOutput' + i + 'Value').html("--");
                        $('#IdAnalogOutput' + i + 'Value').css('color', 'white');
                        $('#IdAnalogOutput' + i + 'Value').css('background-color', 'grey');
                        $('#IdAnalogOutput' + i + 'Value').css('border-color', 'grey');
                        $('#IdAnalogOutput' + i + 'Set').html("--");
                        $('#IdAnalogOutput' + i + 'Set').css("color", "white");
                        $('#IdAnalogOutput' + i + 'Set').css("background-color", 'grey');
                        $('#IdAnalogOutput' + i + 'Set').css("border-color", 'grey');
                    }
                }

                footableTable.trigger('footable_initialized');
                footableTable.trigger('footable_resize');
                footableTable.data('footable').redraw();

            });
        };
    })

        // dynamically adjust Devices display
    .directive('footableDevices', function () {
        return function (scope, element) {
            var footableTable = element.parents('table');// Get the parent of this element
            
            // If this isn't the $last ng-repeat, then ignore
            if (!scope.$last) {
                return false;
            }
            
            // This is the $last ng-repeat, wait and then initialize the footable table
            scope.$evalAsync(function () {

                //console.log("Initializing footableDevices");
                if (!footableTable.hasClass('footable-loaded')) {
                    footableTable.footable({
                        toggleSelector: " > tbody > tr > td > span.footable-toggle"
                    })
                }
                
                //console.log("footableDevices directive: length of scope.Ports=" + scope.Ports.length);
                for (i = 0; i < scope.Ports.length; i++) {
                    //console.log("footableDevices directive: scope.Ports["+i+"].Enabled=" + scope.Ports[i].Enabled);
                    if (scope.Ports[i].Enabled == "YES" && scope.Ports[i].DeviceType != "None") {
                        //$('#IdDevice' + i + 'Status').html(scope.Ports[i].DeviceStatus);
                        $('#IdDevice' + i + 'Status').html("OK");
                        $('#IdDevice' + i + 'Status').css('color', 'white');
                        $('#IdDevice' + i + 'Status').css('background-color', 'green');
                        $('#IdDevice' + i + 'Status').css('border-color', 'green');
                    } else {
                        $('#IdDevice' + i + 'Status').html("--");
                        $('#IdDevice' + i + 'Status').css('color', 'white');
                        $('#IdDevice' + i + 'Status').css('background-color', 'grey');
                        $('#IdDevice' + i + 'Status').css('border-color', 'grey');
                    }
                }

                footableTable.trigger('footable_initialized');
                footableTable.trigger('footable_resize');
                footableTable.data('footable').redraw();
            });
        };
    })

;