﻿/* Controllers */
peakApp
    .controller('ports', [
        '$scope',
        '$http',
        '$modal',
        'socketManager',
        function ($scope, $http, $modal, socketManager) {
            
            // Controller entry
            //#region Controller entry
            $scope.$on('$stateChangeSuccess', function () {
                //console.log("Controller: Ports/Devices");
            });
            $scope.$on('$destroy', function (event) {
                socketManager.removeAllListeners();
            });
            
            $scope.parseInt = parseInt;// This is here so that we can convert to numbers
            //#endregion Controller entry
            
            // Get Ports, Devices and DeviceGroups
            //#region Get Ports, Devices and DeviceGroups
            
            // Retreive PortsArray
            //  /api/ports
            $scope.Ports = [];
            $scope.PortsCount = 0;
            $scope.PortsAllocated = 0;
            $scope.GetPorts = function () {
                //console.log("Inside ports.GetPorts")
                $http({
                    method: "get",
                    url: "/api/ports"
                })
                .success(function (data) {
                        //console.log('ports.GetPorts success - data=' + JSON.stringify(data));
                        $scope.Ports = data;
                        $scope.PortsCount = data.length;
                        
                        $scope.PortsAllocated = 0;
                        for (i = 0; i < $scope.PortsCount; i++) {
                            if (data[i].Device.localeCompare("NONE")) {
                                $scope.PortsAllocated++;
                            }
                        }
                    })
            }
            $scope.GetPorts();
            
            // Retreive Devices
            //  /api/devices
            $scope.Devices = [];
            $scope.DevicesCount = 0;
            $scope.GetDevices = function () {
                //console.log("Inside ports.GetDevices")
                $http({
                    method: "get",
                    url: "/api/devices"
                })
                .success(function (data) {
                        //console.log('ports.GetDevices success - data=' + JSON.stringify(data));
                        $scope.Devices = data;
                        $scope.DevicesCount = data.length;
                    })
            }
            $scope.GetDevices();
            
            // Retreive DeviceGroupsListArray
            //  /api/devicegroups
            $scope.DeviceGroups = [];
            $scope.DeviceGroupsCount = 0;
            $scope.GetDeviceGroups = function () {
                //console.log("Inside ports.GetDeviceGroups")
                $http({
                    method: "get",
                    url: "/api/devicegroups"
                })
                .success(function (data) {
                        //console.log('ports.GetDeviceGroups success - data=' + JSON.stringify(data));
                        $scope.DeviceGroups = data;
                        $scope.DeviceGroupsCount = data.length;
                    })
            }
            $scope.GetDeviceGroups();
            //#endregion Get Ports, Devices and DeviceGroups
            
            // Manage Port Modal
            //#region Manage Port Modal
            $scope.ManagePort = function (currentPort) {
                //console.log("Inside ports.ManagePort")
                var modalInstance = $modal.open({
                    templateUrl: 'views/partials/ManagePort.html',
                    scope : $scope,
                    controller: function ($scope, $modalInstance) {
                        $scope.modalTitle = "Configure Port " + (parseInt(currentPort.Port) + 1);
                        
                        // Create a copy of the current configuration during the configuration process
                        $scope.tempDevice = JSON.parse(JSON.stringify(currentPort));
                        
                        //#region Input options
                        // Name
                        $scope.NamePattern = /^[\w ]{1,15}[\w]{1,1}$/;
                        
                        // DeviceGroup
                        $scope.DeviceGroupOptions = [{ option: 'NONE' }];
                        for (var i = 0; i < $scope.DeviceGroups.length; i++) {
                            $scope.DeviceGroupOptions.push({ option: $scope.DeviceGroups[i] });
                        }
                        //console.log("***** ports.ManagePort: DeviceGroupOptions=" + JSON.stringify($scope.DeviceGroupOptions))
                        
                        // Devices
                        $scope.DeviceOptions = [{ option: 'NONE' }];

                        // Enabled
                        $scope.EnabledOptions = [{ option: 'YES' }, { option: 'NO' }];
                        
                        // Send Events
                        $scope.SendEventsOptions = [{ option: 'YES' }, { option: 'NO' }];
                        //#endregion Input options
                        
                        // Accept changes to the configuration into the temp variable
                        $scope.SetDeviceGroup = function (option) {
                            $scope.tempDevice.DeviceGroup = option;
                            if ($scope.tempDevice.DeviceGroup === "NONE") $scope.tempDevice.Device = "NONE";
                            $scope.DeviceOptions = [{ option: 'NONE' }];
                            for (var i = 0; i < $scope.Devices.length; i++) {
                                if ($scope.Devices[i].group === $scope.tempDevice.DeviceGroup)
                                    $scope.DeviceOptions.push({ option: $scope.Devices[i].id });
                            }
                            //console.log("***** ports.ManagePort: Device Options=" + JSON.stringify($scope.DeviceOptions));
                        }
                        $scope.SetDevice = function (option) {
                            $scope.tempDevice.Device = option;
                        }
                        $scope.SetEnabled = function (option) {
                            if (option === "YES")
                                $scope.tempDevice.Enabled = true;
                            else
                                $scope.tempDevice.Enabled = false;
                        }
                        $scope.SetSendEvents = function (option) {
                            if (option === "YES")
                                $scope.tempDevice.SendEvents = true;
                            else
                                $scope.tempDevice.SendEvents = false;
                        }                        
                        // Apply button press - update the server
                        $scope.apply = function () {
                            //console.log("ports.ManagePort: Sending '/api/ports/" + $scope.tempDevice.Port + " Data=" + JSON.stringify($scope.tempDevice));
                            $http({
                                method: "put",
                                url: "/api/ports/" + $scope.tempDevice.Port,
                                data: $scope.tempDevice
                            }).success(function (ports) {
                                    //console.log('ports.ManagePort:  PUT success data=' + JSON.stringify(ports));
                                    $modalInstance.close(ports);
                                });
                        };
                        
                        $scope.clear = function () {
                            $scope.tempDevice.Name = "Port " + (parseInt($scope.tempDevice.Port)+1);
                            $scope.tempDevice.Device = "NONE";
                            $scope.tempDevice.DeviceGroup = "NONE";
                            $scope.tempDevice.Enabled = false;
                            $scope.tempDevice.SendEvents = false;
                        }
                        
                        // Cancel button press - do nothing
                        $scope.cancel = function () {
                            $modalInstance.dismiss('cancel');
                        };
                    },
                });
                
                // Save the results from the modal to $scope data, so it will automatically update the UI
                modalInstance.result.then(function (ports) {
                    $scope.Ports = ports;
                }, function () {
                    //console.log('ports.ManagePort: Modal dismissed at: ' + new Date());
                });
            };
            //#endregion Manage Port Modal

            //#region Socket messages
            socketManager.on('Ports', function (ports) {
                //console.log('ports.socketManager: Ports=' + JSON.stringify(ports));
                $scope.Ports = ports.Ports;
                $scope.PortsCount = ports.Ports.length;
                
                $scope.PortsAllocated = 0;
                for (i = 0; i < $scope.PortsCount; i++) {
                    if ($scope.Ports[i].Device.localeCompare("NONE")) {
                        $scope.PortsAllocated++;
                    }
                }
            });
            //#endregion Socket messages
        }
    ])

    .controller('port0', [
        '$scope',
        '$http',
        '$modal',
        'socketManager',
        //#region Controller: port 0
        function ($scope, $http, $modal, socketManager) {
            
            // Controller Entry
            //#region Controller Entry
            $scope.$on('$stateChangeSuccess', function () {
                //console.log("Port" + $scope.ThisPort + ".Controller: Entry");
            });
            $scope.$on('$destroy', function (event) {
                socketManager.removeAllListeners();
            });
            //#endregion Controller Entry
            
            // Get Device
            //#region Get Device
            
            // Port and Device variables
            $scope.ThisPort = 0;
            $scope.Ports = [];
            $scope.Device = {};
            
            // Retreive array of Ports
            //  /api/ports
            $scope.GetPorts = function () {
                //console.log("Port" + $scope.ThisPort + ".Controller.GetPorts");
                $http({
                    method: "get",
                    url: "/api/ports"
                })
                .success(function (data) {
                        $scope.Ports = data;
                        $scope.Device = data[$scope.ThisPort];
                        //console.log("Port" + $scope.ThisPort + ".Controller.GetPorts: Device="+JSON.stringify($scope.Device));
                    })
            }
            $scope.GetPorts();
            //#endregion Get Device

            //#region Get Device Information
            // Retreive array of Ports
            //  /api/ports
            $scope.GetDeviceInfo = function () {
                //console.log("Port" + $scope.ThisPort + ".Controller.GetDeviceInfo");
                $http({
                    method: "get",
                    url: "/api/devices/info/" + $scope.ThisPort
                })
                    .success(function (deviceInfo) {
                        $scope.Device.DeviceModel.device = deviceInfo;
                        //console.log("Port" + $scope.ThisPort + ".Controller.GetDeviceInfo: DeviceInfo=" + JSON.stringify(deviceInfo));
                    })
            }
            $scope.GetDeviceInfo();
            //#endregion Get Device Information

            // Get EventLog
            //#region EventLog
            // Retrieve array of Events
            //  /api/eventlog
            $scope.GetEventLog = function () {
                //console.log("Port" + $scope.ThisPort + ".Controller.GetEventLog");
                $http({
                    method: "get",
                    url: "/api/eventlog/"+$scope.ThisPort
                })
                .success(function (eventlog) {
                        $scope.EventLog = eventlog;
                        //console.log("Port" + $scope.ThisPort + ".Controller.GetEventLog: EventLog changed=" + JSON.stringify($scope.EventLog));
                    })
            }
            $scope.GetEventLog();
            //#endregion EventLog

            //===== NonDirectionalPower device =====
            //#region NonDirectionalPower device
            
            //#region NonDirectionalPower ChangeView
            $scope.ChangeFormulaIndex = function (option) {
                //console.log("port0.ChangeFormulaIndex: New formula=" + option);
                for (var i = 0; i < $scope.Device.DeviceModel.parameters.Power.Formulas.length; i++) {
                    if ($scope.Device.DeviceModel.parameters.Power.Formulas[i].Name == option) {
                        $scope.Device.DeviceModel.parameters.Power.FormulaIndex = i;
                        
                        // Persist the change
                        $http({
                            method: "put",
                            url: "/api/devices/" + $scope.Device.DeviceModel.port,
                            data: $scope.Device.DeviceModel
                        }).success(function (updatedmodel) {
                                //console.log("port0.ChangeFormulaIndex: Updated DeviceModel=" + JSON.stringify(updatedmodel));
                                device.DeviceModel = updatedmodel;
                                $modalInstance.close();
                            });
                    }               
                }
            }
            //#endregion NonDirectionalPower ChangeView
            
            //#region Configure device
            $scope.ConfigureNonDirectionalPower = function (port,device) {
                //console.log("Inside ConfigureNonDirectionalPower for port=" + port)
                var modalInstance = $modal.open({
                    templateUrl: 'devices/NonDirectionalPowerConfigure.html',
                    controller: function ($scope, $modalInstance) {

                        //#region ConfigureNonDirectionalPower preamble
                        $scope.modalTitle = "Configure Port " + port + ": " + device.Name;
                        
                        // Setup editing parameters
                        $scope.TempDevice = JSON.parse(JSON.stringify(device));
                        
                        $scope.NamePattern = /^[\w][\w ]{1,15}$/;
                        $scope.LowThresholdPattern = /^(([0-5]){0,1}|([0-4]{0,1}\.[0-9]{0,2})|(5\.[0]{0,2})){0,1}$/;
                        $scope.HighThresholdPattern = /^(([0-5]){0,1}|([0-4]{0,1}\.[0-9]{0,2})|(5\.[0]{0,2})){0,1}$/;
                        $scope.HoldTimeOptions = [{ option: '0' }, { option: '1' }, { option: '2' }, { option: '3' }, { option: '4' }, { option: '5' }];
                        $scope.QualifierOptions = [{ option: 'YES' }, { option: 'NO' }];
                        $scope.ModeOptions = [{ option: 'Normally Open' }, { option: 'Normally Closed' }];
                        //#endregion ConfigureNonDirectionalPower preamble
                        
                        //#region ConfigureNonDirectional set functions
                        $scope.SetPowerHoldTime = function (option) {
                            $scope.TempDevice.DeviceModel.parameters.Power.HoldTime = option;
                        }                 
                        
                        $scope.SetQualifier = function (option) {
                            $scope.TempDevice.DeviceModel.parameters.Power.Qualifier = (option === "YES") ? true:false;
                        }
                        
                        $scope.SetPTTHoldTime = function (option) {
                            $scope.TempDevice.DeviceModel.parameters.PTT.HoldTime = option;
                        }
                        
                        $scope.SetMode = function (option) {
                            $scope.TempDevice.DeviceModel.parameters.PTT.Mode = option;
                        }

                        //#endregion ConfigureNonDirectional set functions
                        
                        //# region ConfigureNonDirectionalPower apply
                        $scope.Apply = function () {
                            //console.log("port0.ConfigureNonDirectionalPower.Apply: Passed DeviceModel=" + JSON.stringify($scope.TempDevice.DeviceModel));
                            
                            $http({
                                method: "put",
                                url: "/api/devices/" + port,
                                data: $scope.TempDevice.DeviceModel
                            }).success(function (updatedmodel) {
                                    //console.log("port0.ConfigureNonDirectionalPower.Apply: Updated DeviceModel=" + JSON.stringify(updatedmodel));
                                    device.DeviceModel = updatedmodel;
                                    $modalInstance.close();
                                });
                        };
                        //# endregion ConfigureNonDirectionalPower preamble
                        
                        //#region ConfigureNonDirectionalPower cancel
                        $scope.Cancel = function () {
                            $modalInstance.dismiss('cancel');
                        };
                        //#endregion ConfigureNonDirectionalPower cancel
                    }
                });
            };
            //#endregion Configure device            
            
            //#region Socket messages
            socketManager.on('NonDirectionalPower', function (message) {
                if (message.Port == $scope.ThisPort) {
                    //console.log("Port" + $scope.ThisPort + ".Controller.SocketOn: NonDirectionalPower=" + JSON.stringify(message));
                    
                    switch (message.Message) {
                        case "Value":
                            switch (message.Parameter.Id) {
                                case "Power":
                                    $scope.Device.DeviceModel.parameters.Power = message.Parameter;
                                    break;

                                case "Qualifier":
                                    $scope.Device.DeviceModel.parameters.PTT = message.Parameter;
                                    break;
                            }
                            break;

                        case "HoldTimeState":
                            if (Message.Parameter.Id == "Qualifier") {
                                $scope.Device.DeviceModel.parameters.PTT.HoldTimeState = message.Parameter.HoldTimeState;
                                $scope.Device.DeviceModel.parameters.PTT.Value = message.Parameter.Value;
                            }
                            break;

                        case "EventLog":
                            $scope.EventLog = message.Parameter;
                            break;
                    }
                }
            });
            //#endregion Socket messages

            //#endregion NonDirectionalPower device

            //===== DirectionalPower+VSWR device =====
            //#region DirectionalPower+VSWR device

            //#region Get Device Information
            // Retreive array of Ports
            //  /api/ports
            $scope.GetDeviceInfo = function () {
                //console.log("Port" + $scope.ThisPort + ".Controller.GetDeviceInfo");
                $http({
                    method: "get",
                    url: "/api/devices/info/" + $scope.ThisPort
                })
                    .success(function (deviceInfo) {
                        $scope.Device.DeviceModel.device = deviceInfo;
                        //console.log("Port" + $scope.ThisPort + ".Controller.GetDeviceInfo: DeviceInfo=" + JSON.stringify(deviceInfo));
                    })
            }
            $scope.GetDeviceInfo();
            //#endregion Get Device Information

            //#region Formula Changes
            //#region ForwardPower Formula Change
            $scope.ChangeForwardPowerFormulaIndex = function (option) {
                //console.log("port" + $scope.ThisPort + ".ChangeForwardPowerFormulaIndex: New formula=" + option);
                for (var i = 0; i < $scope.Device.DeviceModel.parameters.ForwardPower.Formulas.length; i++) {
                    if ($scope.Device.DeviceModel.parameters.ForwardPower.Formulas[i].Name == option) {
                        $scope.Device.DeviceModel.parameters.ForwardPower.FormulaIndex = i;

                        // Persist the change
                        $http({
                            method: "put",
                            url: "/api/devices/" + $scope.Device.DeviceModel.port,
                            data: $scope.Device.DeviceModel
                        }).success(function (updatedmodel) {
                            //console.log("port" + $scope.ThisPort + ".ChangeForwardPowerFormulaIndex: Updated DeviceModel=" + JSON.stringify(updatedmodel));
                            device.DeviceModel = updatedmodel;
                            $modalInstance.close();
                        });
                    }
                }
            }
            //#endregion ForwardPower Formula Change

            //#region ReflectedPower Formula Change
            $scope.ChangeReflectedPowerFormulaIndex = function (option) {
                //console.log("port" + $scope.ThisPort + ".ChangeReflectedPowerFormulaIndex: New formula=" + option);
                for (var i = 0; i < $scope.Device.DeviceModel.parameters.ReflectedPower.Formulas.length; i++) {
                    if ($scope.Device.DeviceModel.parameters.ReflectedPower.Formulas[i].Name == option) {
                        $scope.Device.DeviceModel.parameters.ReflectedPower.FormulaIndex = i;

                        // Persist the change
                        $http({
                            method: "put",
                            url: "/api/devices/" + $scope.Device.DeviceModel.port,
                            data: $scope.Device.DeviceModel
                        }).success(function (updatedmodel) {
                            //console.log("port" + $scope.ThisPort + ".ChangeReflectedPowerFormulaIndex: Updated DeviceModel=" + JSON.stringify(updatedmodel));
                            device.DeviceModel = updatedmodel;
                            $modalInstance.close();
                        });
                    }
                }
            }
            //#endregion ReflectedPower Formula Change

            //#region Temperature Formula Change
            $scope.ChangeTemperatureFormulaIndex = function (option) {
                //console.log("port" + $scope.ThisPort + ".ChangeTemperatureFormulaIndex: New formula=" + option);
                for (var i = 0; i < $scope.Device.DeviceModel.parameters.Temperature.Formulas.length; i++) {
                    if ($scope.Device.DeviceModel.parameters.Temperature.Formulas[i].Name == option) {
                        $scope.Device.DeviceModel.parameters.Temperature.FormulaIndex = i;

                        // Persist the change
                        $http({
                            method: "put",
                            url: "/api/devices/" + $scope.Device.DeviceModel.port,
                            data: $scope.Device.DeviceModel
                        }).success(function (updatedmodel) {
                            //console.log("port" + $scope.ThisPort + ".ChangeTemperatureFormulaIndex: Updated DeviceModel=" + JSON.stringify(updatedmodel));
                            device.DeviceModel = updatedmodel;
                            $modalInstance.close();
                        });
                    }
                }
            }
            //#endregion Temperature Formula Change
            //#endregion Formula Changes

            //#region Socket messages
            socketManager.on('DirectionalPower+VSWR', function (message) {
                if (message.Port == $scope.ThisPort) {
                    //console.log("Port" + $scope.ThisPort + ".Controller.SocketOn: DirectionalPower+VSWR=" + JSON.stringify(message));

                    switch (message.Message) {
                        case "DeviceInformation":
                            $scope.Device.DeviceModel.device = message.device;
                            break;

                        case "ForwardPower":
                            $scope.Device.DeviceModel.parameters.ForwardPower = message.Parameter.ForwardPower;
                            break;

                        case "ReflectedPower":
                            $scope.Device.DeviceModel.parameters.ReflectedPower = message.Parameter.ReflectedPower;
                            break;

                        case "VSWR":
                            $scope.Device.DeviceModel.parameters.VSWR = message.Parameter.VSWR;
                            break;

                        case "Temperature":
                            $scope.Device.DeviceModel.parameters.Temperature = message.Parameter.Temperature;
                            break;
                    }
                }
            });
            //#endregion Socket messages
            //#endregion DirectionalPower+VSWR device

            //===== TempHumidity sensor =====
            //#region TempHumiditySensor device
            //#region Configure device
            $scope.TempHumiditySensorConfiguration = function (device,parameterIndex) {
                //console.log("Inside TempHumiditySensorConfiguration for port=" + device.port)
                var modalInstance = $modal.open({
                    templateUrl: 'devices/TempHumiditySensorConfiguration.html',
                    scope: $scope,
                    controller: function ($scope, $modalInstance) {

                        //#region TempHumiditySensorConfiguration preamble
                        $scope.modalTitle = "Configure Port " + device.port + ": " + device.title;
                        $scope.parameterIndex = parameterIndex;
                        $scope.TempDevice = JSON.parse(JSON.stringify(device));
                        $scope.RangedLowThresholdPattern = /^([0-5]{1}|[0-4]?\.[0-9]{0,2}|5\.[0]{0,2})$/;
                        $scope.RangedHighThresholdPattern = /^([0-5]{1}|[0-4]?\.[0-9]{0,2}|5\.[0]{0,2})$/;
                        $scope.HoldTimeOptions = [{ option: '0' }, { option: '1' }, { option: '2' }, { option: '3' }, { option: '4' }, { option: '5' }];
                        //#endregion TempHumiditySensorConfiguration preamble

                        //#region TempHumiditySensorConfiguration set functions
                        $scope.SetHoldTime = function (option) {
                            $scope.TempDevice.parameters[$scope.parameterIndex].HoldTime = Number(option);
                        }

                        $scope.SelectFormula = function (option) {
                            $scope.TempDevice.parameters[$scope.parameterIndex].FormulaIndex = option;
                        }

                        $scope.Recalculate = function (rangedLow, rangedHigh, formulaIndex) {
                            var formulaCalcRequest = {
                                parmeterIndex: $scope.parameterIndex,
                                rangedLow: rangedLow,
                                rangedHigh: rangedHigh,
                                formulaLow: -1,
                                formulaHigh: -1,
                                formulaIndex: formulaIndex
                            }

                            $http({
                                method: "put",
                                url: "/api/devices/formula/" + device.port,
                                data: formulaCalcRequest
                            }).success(function (formulaCalcReturn) {
                                //console.log("port0.TempHumiditySensorConfiguration.Apply: Calculated formula value=" + JSON.stringify(formulaCalcReturn));
                                $scope.TempDevice.parameters[$scope.parameterIndex].Formulas[formulaIndex].FormulaLowThreshold = formulaCalcReturn.formulaLow;
                                $scope.TempDevice.parameters[$scope.parameterIndex].Formulas[formulaIndex].FormulaHighThreshold = formulaCalcReturn.formulaHigh;
                            });
                        }
                        //#endregion TempHumiditySensorConfiguration set functions

                        //#region TempHumiditySensorConfiguration apply
                        $scope.Apply = function () {
                            //console.log("port0.TempHumiditySensorConfiguration.Apply: Device to update=" + JSON.stringify($scope.TempDevice));
                            $scope.TempDevice.parameters[$scope.parameterIndex].RangedLowThreshold = Number($scope.TempDevice.parameters[$scope.parameterIndex].RangedLowThreshold).toFixed(2);
                            $scope.TempDevice.parameters[$scope.parameterIndex].RangedHighThreshold = Number($scope.TempDevice.parameters[$scope.parameterIndex].RangedHighThreshold).toFixed(2);

                            $http({
                                method: "put",
                                url: "/api/devices/" + device.port + "/" + $scope.parameterIndex,
                                data: $scope.TempDevice
                            }).success(function (updatedDevice) {
                                //console.log("port0.TempHumiditySensorConfiguration.Apply: Updated Device=" + JSON.stringify(updatedDevice));
                                $scope.Device.DeviceModel = updatedDevice;
                                $modalInstance.close();
                            });
                        };
                        //#endregion TempHumiditySensorConfiguration apply

                        //#region TempHumiditySensorConfiguration cancel
                        $scope.Cancel = function () {
                            $modalInstance.dismiss('cancel');
                        };
                        //#endregion TempHumiditySensorConfiguration cancel
                    }
                });
            };
            //#endregion Configure device            

            //#region Socket messages
            socketManager.on('TempHumiditySensor', function (message) {
                if (message.Port == $scope.ThisPort) {
                    //console.log("Port" + $scope.ThisPort + ".Controller.SocketOn: TempHumiditySensor=" + JSON.stringify(message));

                    switch (message.Message) {
                        case "Value":
                            switch (message.Parameter.Id) {
                                case "Temperature":
                                    $scope.Device.DeviceModel.parameters[1] = message.Parameter;
                                    break;

                                case "Humidity":
                                    $scope.Device.DeviceModel.parameters[0] = message.Parameter;
                                    break;
                            }
                            break;

                        case "EventLog":
                            $scope.EventLog = message.Parameter;
                            break;
                    }
                }
            });
            //#endregion Socket messages
            //#endregion TempHumiditySensor device

            //===== DoorSensor device =====
            //#region DoorSensor device
            //#region Configure device
            $scope.DoorSensorConfiguration = function (device) {
                //console.log("Inside DoorSensorConfiguration for port=" + device.port)
                var modalInstance = $modal.open({
                    templateUrl: 'devices/DoorSensorConfiguration.html',
                    scope: $scope,
                    controller: function ($scope, $modalInstance) {

                        //#region DoorSensorConfiguration preamble
                        $scope.modalTitle = "Configure Port " + device.port + ": " + device.title;
                        $scope.TempDevice = JSON.parse(JSON.stringify(device));
                        $scope.ModeOptions = [{ option: 'Normally Open' }, { option:'Normally Closed'}];
                        $scope.HoldTimeOptions = [{ option: '0' }, { option: '1' }, { option: '2' }, { option: '3' }, { option: '4' }, { option: '5' }];
                        //#endregion DoorSensorConfiguration preamble

                        //#region DoorSensorConfiguration set functions
                        $scope.SetHoldTime = function (option) {
                            $scope.TempDevice.parameters[0].HoldTime = Number(option);
                        }
                        $scope.SetMode = function (option) {
                            $scope.TempDevice.parameters[0].Mode = option;
                        }
                        //#endregion DoorSensorConfiguration set functions

                        //#region DoorSensorConfiguration apply
                        $scope.Apply = function () {
                            $http({
                                method: "put",
                                url: "/api/devices/" + device.port + "/0",// Just one parameter to worry about
                                data: $scope.TempDevice
                            }).success(function (updatedDevice) {
                                //console.log("port0.DoorSensorConfiguration.Apply: Updated Device=" + JSON.stringify(updatedDevice));
                                $scope.Device.DeviceModel = updatedDevice;
                                $modalInstance.close();
                            });
                        };
                        //#endregion DoorSensorConfiguration apply

                        //#region DoorSensorConfiguration cancel
                        $scope.Cancel = function () {
                            $modalInstance.dismiss('cancel');
                        };
                        //#endregion DoorSensorConfiguration cancel
                    }
                });
            };
            //#endregion Configure device            

            //#region Socket messages
            socketManager.on('DoorSensor', function (message) {
                if (message.Port == $scope.ThisPort) {
                    switch (message.Message) {
                        case "Value":
                            switch (message.Parameter.Id) {
                                case "Door":
                                    //console.log("Port" + $scope.ThisPort + ".Controller.SocketOn: DoorSensor=" + JSON.stringify(message));
                                    $scope.Device.DeviceModel.parameters[0] = message.Parameter;
                                    break;
                            }
                            break;

                        case "EventLog":
                            $scope.EventLog = message.Parameter;
                            break;
                    }
                }
            });
            //#endregion Socket messages
            //#endregion DoorSensor device

            //===== ACSensor device =====
            //#region ACSensor device
            //#region Configure device
            $scope.ACSensorConfiguration = function (device) {
                //console.log("Inside ACSensorConfiguration for port=" + device.port)
                var modalInstance = $modal.open({
                    templateUrl: 'devices/ACSensorConfiguration.html',
                    scope: $scope,
                    controller: function ($scope, $modalInstance) {

                        //#region ACSensorConfiguration preamble
                        $scope.modalTitle = "Configure Port " + device.port + ": " + device.title;
                        $scope.TempDevice = JSON.parse(JSON.stringify(device));
                        $scope.RangedLowThresholdPattern = /^([0-5]{1}|[0-4]?\.[0-9]{0,2}|5\.[0]{0,2})$/;
                        $scope.RangedHighThresholdPattern = /^([0-5]{1}|[0-4]?\.[0-9]{0,2}|5\.[0]{0,2})$/;
                        $scope.HoldTimeOptions = [{ option: '0' }, { option: '1' }, { option: '2' }, { option: '3' }, { option: '4' }, { option: '5' }];
                        //#endregion ACSensorConfiguration preamble

                        //#region ACSensorConfiguration set functions
                        $scope.SetHoldTime = function (option) {
                            $scope.TempDevice.parameters[0].HoldTime = Number(option);
                        }

                        $scope.SelectFormula = function (option) {
                            $scope.TempDevice.parameters[0].FormulaIndex = option;
                        }

                        $scope.Recalculate = function (rangedLow, rangedHigh, formulaIndex) {
                            var formulaCalcRequest = {
                                parmeterIndex: 0,
                                rangedLow: rangedLow,
                                rangedHigh: rangedHigh,
                                formulaLow: -1,
                                formulaHigh: -1,
                                formulaIndex: formulaIndex
                            }

                            $http({
                                method: "put",
                                url: "/api/devices/formula/" + device.port,
                                data: formulaCalcRequest
                            }).success(function (formulaCalcReturn) {
                                //console.log("port0.ACSensorConfiguration.Apply: Calculated formula value=" + JSON.stringify(formulaCalcReturn));
                                $scope.TempDevice.parameters[0].Formulas[formulaIndex].FormulaLowThreshold = formulaCalcReturn.formulaLow;
                                $scope.TempDevice.parameters[0].Formulas[formulaIndex].FormulaHighThreshold = formulaCalcReturn.formulaHigh;
                            });
                        }
                        //#endregion ACSensorConfiguration set functions

                        //#region ACSensorConfiguration apply
                        $scope.Apply = function () {
                            //console.log("port0.ACSensorConfiguration.Apply: Device to update=" + JSON.stringify($scope.TempDevice));
                            $scope.TempDevice.parameters[0].RangedLowThreshold = Number($scope.TempDevice.parameters[0].RangedLowThreshold).toFixed(2);
                            $scope.TempDevice.parameters[0].RangedHighThreshold = Number($scope.TempDevice.parameters[0].RangedHighThreshold).toFixed(2);

                            $http({
                                method: "put",
                                url: "/api/devices/" + device.port + "/0",// Default to parameter 0 
                                data: $scope.TempDevice
                            }).success(function (updatedDevice) {
                                //console.log("port0.ACSensorConfiguration.Apply: Updated Device=" + JSON.stringify(updatedDevice));
                                $scope.Device.DeviceModel = updatedDevice;
                                $modalInstance.close();
                            });
                        };
                        //#endregion ACSensorConfiguration apply

                        //#region ACSensorConfiguration cancel
                        $scope.Cancel = function () {
                            $modalInstance.dismiss('cancel');
                        };
                        //#endregion ACSensorConfiguration cancel
                    }
                });
            };
            //#endregion Configure device            

            //#region Socket messages
            socketManager.on('ACSensor', function (message) {
                if (message.Port == $scope.ThisPort) {
                    //console.log("Port" + $scope.ThisPort + ".Controller.SocketOn: ACSensor=" + JSON.stringify(message));

                    switch (message.Message) {
                        case "Value":
                            switch (message.Parameter.Id) {
                                case "AC":
                                    $scope.Device.DeviceModel.parameters[0] = message.Parameter;
                                    break;
                            }
                            break;

                        case "EventLog":
                            $scope.EventLog = message.Parameter;
                            break;                    }
                }
            });
            //#endregion Socket messages
            //#endregion ACSensor device
        }
        //#endregion Controller: port 0
    ])

    .controller('port1', [
        '$scope',
        '$http',
        '$modal',
        'socketManager',
        //#region Controller: port 1
        function ($scope, $http, $modal, socketManager) {
            
            // Controller Entry
            //#region Controller Entry
            $scope.$on('$stateChangeSuccess', function () {
                //console.log("Port" + $scope.ThisPort + ".Controller: Entry");
            });
            $scope.$on('$destroy', function (event) {
                socketManager.removeAllListeners();
            });
            //#endregion Controller Entry
            
            // Get Device
            //#region Get Device
            
            // Port and Device variables
            $scope.ThisPort = 1;
            $scope.Ports = [];
            $scope.Device = {};
            
            // Retreive array of Ports
            //  /api/ports
            $scope.GetPorts = function () {
                //console.log("Port" + $scope.ThisPort + ".Controller.GetPorts");
                $http({
                    method: "get",
                    url: "/api/ports"
                })
                .success(function (data) {
                        $scope.Ports = data;
                        $scope.Device = data[$scope.ThisPort];
                        //console.log("Port" + $scope.ThisPort + ".Controller.GetPorts: Device=" + JSON.stringify($scope.Device));
                    })
            }
            $scope.GetPorts();
            //#endregion Get Device
            
            // Get EventLog
            //#region EventLog
            // Retrieve array of Events
            //  /api/eventlog
            $scope.GetEventLog = function () {
                //console.log("Port" + $scope.ThisPort + ".Controller.GetEventLog");
                $http({
                    method: "get",
                    url: "/api/eventlog/" + $scope.ThisPort
                })
                .success(function (eventlog) {
                        $scope.EventLog = eventlog;
                        //console.log("Port" + $scope.ThisPort + ".Controller.GetEventLog: EventLog changed=" + JSON.stringify($scope.EventLog));
                    })
            }
            $scope.GetEventLog();
            //#endregion EventLog
            
            //===== NonDirectionalPower device =====
            //#region NonDirectionalPower device
            
            //#region NonDirectionalPower ChangeView
            $scope.ChangeFormulaIndex = function (option) {
                //console.log("port0.ChangeFormulaIndex: New formula=" + option);
                for (var i = 0; i < $scope.Device.DeviceModel.parameters.Power.Formulas.length; i++) {
                    if ($scope.Device.DeviceModel.parameters.Power.Formulas[i].Name == option) {
                        $scope.Device.DeviceModel.parameters.Power.FormulaIndex = i;
                        
                        // Persist the change
                        $http({
                            method: "put",
                            url: "/api/devices/" + $scope.Device.DeviceModel.port,
                            data: $scope.Device.DeviceModel
                        }).success(function (updatedmodel) {
                                //console.log("port0.ChangeFormulaIndex: Updated DeviceModel=" + JSON.stringify(updatedmodel));
                                device.DeviceModel = updatedmodel;
                                $modalInstance.close();
                            });
                    }
                }
            }
            //#endregion NonDirectionalPower ChangeView
            
            //#region Configure device
            $scope.ConfigureNonDirectionalPower = function (port, device) {
                //console.log("Inside ConfigureNonDirectionalPower for port=" + port)
                var modalInstance = $modal.open({
                    templateUrl: 'devices/NonDirectionalPowerConfigure.html',
                    controller: function ($scope, $modalInstance) {
                        
                        //#region ConfigureNonDirectionalPower preamble
                        $scope.modalTitle = "Configure Port " + port + ": " + device.Name;
                        
                        // Setup editing parameters
                        $scope.TempDevice = JSON.parse(JSON.stringify(device));
                        
                        $scope.NamePattern = /^[\w][\w ]{1,15}$/;
                        $scope.LowThresholdPattern = /^(([0-5]){0,1}|([0-4]{0,1}\.[0-9]{0,2})|(5\.[0]{0,2})){0,1}$/;
                        $scope.HighThresholdPattern = /^(([0-5]){0,1}|([0-4]{0,1}\.[0-9]{0,2})|(5\.[0]{0,2})){0,1}$/;
                        $scope.HoldTimeOptions = [{ option: '0' }, { option: '1' }, { option: '2' }, { option: '3' }, { option: '4' }, { option: '5' }];
                        $scope.QualifierOptions = [{ option: 'YES' }, { option: 'NO' }];
                        $scope.ModeOptions = [{ option: 'Normally Open' }, { option: 'Normally Closed' }];
                        //#endregion ConfigureNonDirectionalPower preamble
                        
                        //#region ConfigureNonDirectional set functions
                        $scope.SetPowerHoldTime = function (option) {
                            $scope.TempDevice.DeviceModel.parameters.Power.HoldTime = option;
                        }
                        
                        $scope.SetQualifier = function (option) {
                            $scope.TempDevice.DeviceModel.parameters.Power.Qualifier = (option === "YES") ? true:false;
                        }
                        
                        $scope.SetPTTHoldTime = function (option) {
                            $scope.TempDevice.DeviceModel.parameters.PTT.HoldTime = option;
                        }
                        
                        $scope.SetMode = function (option) {
                            $scope.TempDevice.DeviceModel.parameters.PTT.Mode = option;
                        }
                        
                        //#endregion ConfigureNonDirectional set functions
                        
                        //# region ConfigureNonDirectionalPower apply
                        $scope.Apply = function () {
                            //console.log("port0.ConfigureNonDirectionalPower.Apply: Passed DeviceModel=" + JSON.stringify($scope.TempDevice.DeviceModel));
                            
                            $http({
                                method: "put",
                                url: "/api/devices/" + port,
                                data: $scope.TempDevice.DeviceModel
                            }).success(function (updatedmodel) {
                                    //console.log("port0.ConfigureNonDirectionalPower.Apply: Updated DeviceModel=" + JSON.stringify(updatedmodel));
                                    device.DeviceModel = updatedmodel;
                                    $modalInstance.close();
                                });
                        };
                        //# endregion ConfigureNonDirectionalPower preamble
                        
                        //#region ConfigureNonDirectionalPower cancel
                        $scope.Cancel = function () {
                            $modalInstance.dismiss('cancel');
                        };
                        //#endregion ConfigureNonDirectionalPower cancel
                    }
                });
            };
            //#endregion Configure device            
            
            //#region Socket messages
            socketManager.on('NonDirectionalPower', function (message) {
                if (message.Port == $scope.ThisPort) {
                    //console.log("Port" + $scope.ThisPort + ".Controller.SocketOn: NonDirectionalPower=" + JSON.stringify(message));
                    
                    switch (message.Message) {
                        case "Value":
                            switch (message.Parameter.Id) {
                                case "Power":
                                    $scope.Device.DeviceModel.parameters.Power = message.Parameter;
                                    break;

                                case "Qualifier":
                                    $scope.Device.DeviceModel.parameters.PTT = message.Parameter;
                                    break;
                            }
                            break;

                        case "HoldTimeState":
                            if (Message.Parameter.Id == "Qualifier") {
                                $scope.Device.DeviceModel.parameters.PTT.HoldTimeState = message.Parameter.HoldTimeState;
                                $scope.Device.DeviceModel.parameters.PTT.Value = message.Parameter.Value;
                            }
                            break;

                        case "EventLog":
                            $scope.EventLog = message.Parameter;
                            break;
                    }
                }
            });
            //#endregion Socket messages

            //#endregion NonDirectionalPower device

            //===== DirectionalPower+VSWR device =====
            //#region DirectionalPower+VSWR device

            //#region Get Device Information
            // Retreive array of Ports
            //  /api/ports
            $scope.GetDeviceInfo = function () {
                //console.log("Port" + $scope.ThisPort + ".Controller.GetDeviceInfo");
                $http({
                    method: "get",
                    url: "/api/devices/info/" + $scope.ThisPort
                })
                    .success(function (deviceInfo) {
                        $scope.Device.DeviceModel.device = deviceInfo;
                        //console.log("Port" + $scope.ThisPort + ".Controller.GetDeviceInfo: DeviceInfo=" + JSON.stringify(deviceInfo));
                    })
            }
            $scope.GetDeviceInfo();
            //#endregion Get Device Information

            //#region Formula Changes
            //#region ForwardPower Formula Change
            $scope.ChangeForwardPowerFormulaIndex = function (option) {
                //console.log("port" + $scope.ThisPort + ".ChangeForwardPowerFormulaIndex: New formula=" + option);
                for (var i = 0; i < $scope.Device.DeviceModel.parameters.ForwardPower.Formulas.length; i++) {
                    if ($scope.Device.DeviceModel.parameters.ForwardPower.Formulas[i].Name == option) {
                        $scope.Device.DeviceModel.parameters.ForwardPower.FormulaIndex = i;

                        // Persist the change
                        $http({
                            method: "put",
                            url: "/api/devices/" + $scope.Device.DeviceModel.port,
                            data: $scope.Device.DeviceModel
                        }).success(function (updatedmodel) {
                            //console.log("port" + $scope.ThisPort + ".ChangeForwardPowerFormulaIndex: Updated DeviceModel=" + JSON.stringify(updatedmodel));
                            device.DeviceModel = updatedmodel;
                            $modalInstance.close();
                        });
                    }
                }
            }
            //#endregion ForwardPower Formula Change

            //#region ReflectedPower Formula Change
            $scope.ChangeReflectedPowerFormulaIndex = function (option) {
                //console.log("port" + $scope.ThisPort + ".ChangeReflectedPowerFormulaIndex: New formula=" + option);
                for (var i = 0; i < $scope.Device.DeviceModel.parameters.ReflectedPower.Formulas.length; i++) {
                    if ($scope.Device.DeviceModel.parameters.ReflectedPower.Formulas[i].Name == option) {
                        $scope.Device.DeviceModel.parameters.ReflectedPower.FormulaIndex = i;

                        // Persist the change
                        $http({
                            method: "put",
                            url: "/api/devices/" + $scope.Device.DeviceModel.port,
                            data: $scope.Device.DeviceModel
                        }).success(function (updatedmodel) {
                            //console.log("port" + $scope.ThisPort + ".ChangeReflectedPowerFormulaIndex: Updated DeviceModel=" + JSON.stringify(updatedmodel));
                            device.DeviceModel = updatedmodel;
                            $modalInstance.close();
                        });
                    }
                }
            }
            //#endregion ReflectedPower Formula Change

            //#region Temperature Formula Change
            $scope.ChangeTemperatureFormulaIndex = function (option) {
                //console.log("port" + $scope.ThisPort + ".ChangeTemperatureFormulaIndex: New formula=" + option);
                for (var i = 0; i < $scope.Device.DeviceModel.parameters.Temperature.Formulas.length; i++) {
                    if ($scope.Device.DeviceModel.parameters.Temperature.Formulas[i].Name == option) {
                        $scope.Device.DeviceModel.parameters.Temperature.FormulaIndex = i;

                        // Persist the change
                        $http({
                            method: "put",
                            url: "/api/devices/" + $scope.Device.DeviceModel.port,
                            data: $scope.Device.DeviceModel
                        }).success(function (updatedmodel) {
                            //console.log("port" + $scope.ThisPort + ".ChangeTemperatureFormulaIndex: Updated DeviceModel=" + JSON.stringify(updatedmodel));
                            device.DeviceModel = updatedmodel;
                            $modalInstance.close();
                        });
                    }
                }
            }
            //#endregion Temperature Formula Change
            //#endregion Formula Changes


            //#region Socket messages
            socketManager.on('DirectionalPower+VSWR', function (message) {
                if (message.Port == $scope.ThisPort) {
                    //console.log("Port" + $scope.ThisPort + ".Controller.SocketOn: DirectionalPower+VSWR=" + JSON.stringify(message));

                    switch (message.Message) {
                        case "DeviceInformation":
                            $scope.Device.DeviceModel.device = message.device;
                            break;

                        case "ForwardPower":
                            $scope.Device.DeviceModel.parameters.ForwardPower = message.Parameter.ForwardPower;
                            break;

                        case "ReflectedPower":
                            $scope.Device.DeviceModel.parameters.ReflectedPower = message.Parameter.ReflectedPower;
                            break;

                        case "VSWR":
                            $scope.Device.DeviceModel.parameters.VSWR = message.Parameter.VSWR;
                            break;

                        case "Temperature":
                            $scope.Device.DeviceModel.parameters.Temperature = message.Parameter.Temperature;
                            break;
                    }
                }
            });
            //#endregion Socket messages
            //#endregion DirectionalPower+VSWR device

            //===== TempHumidity sensor =====
            //#region TempHumiditySensor device
            //#region Configure device
            $scope.TempHumiditySensorConfiguration = function (device, parameterIndex) {
                //console.log("Inside TempHumiditySensorConfiguration for port=" + device.port)
                var modalInstance = $modal.open({
                    templateUrl: 'devices/TempHumiditySensorConfiguration.html',
                    scope: $scope,
                    controller: function ($scope, $modalInstance) {

                        //#region TempHumiditySensorConfiguration preamble
                        $scope.modalTitle = "Configure Port " + device.port + ": " + device.title;
                        $scope.parameterIndex = parameterIndex;
                        $scope.TempDevice = JSON.parse(JSON.stringify(device));
                        $scope.RangedLowThresholdPattern = /^([0-5]{1}|[0-4]?\.[0-9]{0,2}|5\.[0]{0,2})$/;
                        $scope.RangedHighThresholdPattern = /^([0-5]{1}|[0-4]?\.[0-9]{0,2}|5\.[0]{0,2})$/;
                        $scope.HoldTimeOptions = [{ option: '0' }, { option: '1' }, { option: '2' }, { option: '3' }, { option: '4' }, { option: '5' }];
                        //#endregion TempHumiditySensorConfiguration preamble

                        //#region TempHumiditySensorConfiguration set functions
                        $scope.SetHoldTime = function (option) {
                            $scope.TempDevice.parameters[$scope.parameterIndex].HoldTime = Number(option);
                        }

                        $scope.SelectFormula = function (option) {
                            $scope.TempDevice.parameters[$scope.parameterIndex].FormulaIndex = option;
                        }

                        $scope.Recalculate = function (rangedLow, rangedHigh, formulaIndex) {
                            var formulaCalcRequest = {
                                parmeterIndex: $scope.parameterIndex,
                                rangedLow: rangedLow,
                                rangedHigh: rangedHigh,
                                formulaLow: -1,
                                formulaHigh: -1,
                                formulaIndex: formulaIndex
                            }

                            $http({
                                method: "put",
                                url: "/api/devices/formula/" + device.port,
                                data: formulaCalcRequest
                            }).success(function (formulaCalcReturn) {
                                //console.log("port0.TempHumiditySensorConfiguration.Apply: Calculated formula value=" + JSON.stringify(formulaCalcReturn));
                                $scope.TempDevice.parameters[$scope.parameterIndex].Formulas[formulaIndex].FormulaLowThreshold = formulaCalcReturn.formulaLow;
                                $scope.TempDevice.parameters[$scope.parameterIndex].Formulas[formulaIndex].FormulaHighThreshold = formulaCalcReturn.formulaHigh;
                            });
                        }
                        //#endregion TempHumiditySensorConfiguration set functions

                        //#region TempHumiditySensorConfiguration apply
                        $scope.Apply = function () {
                            //console.log("port0.TempHumiditySensorConfiguration.Apply: Device to update=" + JSON.stringify($scope.TempDevice));
                            $scope.TempDevice.parameters[$scope.parameterIndex].RangedLowThreshold = Number($scope.TempDevice.parameters[$scope.parameterIndex].RangedLowThreshold).toFixed(2);
                            $scope.TempDevice.parameters[$scope.parameterIndex].RangedHighThreshold = Number($scope.TempDevice.parameters[$scope.parameterIndex].RangedHighThreshold).toFixed(2);

                            $http({
                                method: "put",
                                url: "/api/devices/" + device.port + "/" + $scope.parameterIndex,
                                data: $scope.TempDevice
                            }).success(function (updatedDevice) {
                                //console.log("port0.TempHumiditySensorConfiguration.Apply: Updated Device=" + JSON.stringify(updatedDevice));
                                $scope.Device.DeviceModel = updatedDevice;
                                $modalInstance.close();
                            });
                        };
                        //#endregion TempHumiditySensorConfiguration apply

                        //#region TempHumiditySensorConfiguration cancel
                        $scope.Cancel = function () {
                            $modalInstance.dismiss('cancel');
                        };
                        //#endregion TempHumiditySensorConfiguration cancel
                    }
                });
            };
            //#endregion Configure device            

            //#region Socket messages
            socketManager.on('TempHumiditySensor', function (message) {
                if (message.Port == $scope.ThisPort) {
                    //console.log("Port" + $scope.ThisPort + ".Controller.SocketOn: TempHumiditySensor=" + JSON.stringify(message));

                    switch (message.Message) {
                        case "Value":
                            switch (message.Parameter.Id) {
                                case "Temperature":
                                    $scope.Device.DeviceModel.parameters[1] = message.Parameter;
                                    break;

                                case "Humidity":
                                    $scope.Device.DeviceModel.parameters[0] = message.Parameter;
                                    break;
                            }
                            break;

                        case "EventLog":
                            $scope.EventLog = message.Parameter;
                            break;
                    }
                }
            });
            //#endregion Socket messages
            //#endregion TempHumiditySensor device

            //===== DoorSensor device =====
            //#region DoorSensor device
            //#region Configure device
            $scope.DoorSensorConfiguration = function (device) {
                //console.log("Inside DoorSensorConfiguration for port=" + device.port)
                var modalInstance = $modal.open({
                    templateUrl: 'devices/DoorSensorConfiguration.html',
                    scope: $scope,
                    controller: function ($scope, $modalInstance) {

                        //#region DoorSensorConfiguration preamble
                        $scope.modalTitle = "Configure Port " + device.port + ": " + device.title;
                        $scope.TempDevice = JSON.parse(JSON.stringify(device));
                        $scope.ModeOptions = [{ option: 'Normally Open' }, { option: 'Normally Closed' }];
                        $scope.HoldTimeOptions = [{ option: '0' }, { option: '1' }, { option: '2' }, { option: '3' }, { option: '4' }, { option: '5' }];
                        //#endregion DoorSensorConfiguration preamble

                        //#region DoorSensorConfiguration set functions
                        $scope.SetHoldTime = function (option) {
                            $scope.TempDevice.parameters[0].HoldTime = Number(option);
                        }
                        $scope.SetMode = function (option) {
                            $scope.TempDevice.parameters[0].Mode = option;
                        }
                        //#endregion DoorSensorConfiguration set functions

                        //#region DoorSensorConfiguration apply
                        $scope.Apply = function () {
                            $http({
                                method: "put",
                                url: "/api/devices/" + device.port + "/0",// Just one parameter to worry about
                                data: $scope.TempDevice
                            }).success(function (updatedDevice) {
                                //console.log("port0.DoorSensorConfiguration.Apply: Updated Device=" + JSON.stringify(updatedDevice));
                                $scope.Device.DeviceModel = updatedDevice;
                                $modalInstance.close();
                            });
                        };
                        //#endregion DoorSensorConfiguration apply

                        //#region DoorSensorConfiguration cancel
                        $scope.Cancel = function () {
                            $modalInstance.dismiss('cancel');
                        };
                        //#endregion DoorSensorConfiguration cancel
                    }
                });
            };
            //#endregion Configure device            

            //#region Socket messages
            socketManager.on('DoorSensor', function (message) {
                if (message.Port == $scope.ThisPort) {
                    switch (message.Message) {
                        case "Value":
                            switch (message.Parameter.Id) {
                                case "Door":
                                    //console.log("Port" + $scope.ThisPort + ".Controller.SocketOn: DoorSensor=" + JSON.stringify(message));
                                    $scope.Device.DeviceModel.parameters[0] = message.Parameter;
                                    break;
                            }
                            break;

                        case "EventLog":
                            $scope.EventLog = message.Parameter;
                            break;
                    }
                }
            });
            //#endregion Socket messages
            //#endregion DoorSensor device

            //===== ACSensor device =====
            //#region ACSensor device
            //#region Configure device
            $scope.ACSensorConfiguration = function (device) {
                //console.log("Inside ACSensorConfiguration for port=" + device.port)
                var modalInstance = $modal.open({
                    templateUrl: 'devices/ACSensorConfiguration.html',
                    scope: $scope,
                    controller: function ($scope, $modalInstance) {

                        //#region ACSensorConfiguration preamble
                        $scope.modalTitle = "Configure Port " + device.port + ": " + device.title;
                        $scope.TempDevice = JSON.parse(JSON.stringify(device));
                        $scope.RangedLowThresholdPattern = /^([0-5]{1}|[0-4]?\.[0-9]{0,2}|5\.[0]{0,2})$/;
                        $scope.RangedHighThresholdPattern = /^([0-5]{1}|[0-4]?\.[0-9]{0,2}|5\.[0]{0,2})$/;
                        $scope.HoldTimeOptions = [{ option: '0' }, { option: '1' }, { option: '2' }, { option: '3' }, { option: '4' }, { option: '5' }];
                        //#endregion ACSensorConfiguration preamble

                        //#region ACSensorConfiguration set functions
                        $scope.SetHoldTime = function (option) {
                            $scope.TempDevice.parameters[0].HoldTime = Number(option);
                        }

                        $scope.SelectFormula = function (option) {
                            $scope.TempDevice.parameters[0].FormulaIndex = option;
                        }

                        $scope.Recalculate = function (rangedLow, rangedHigh, formulaIndex) {
                            var formulaCalcRequest = {
                                parmeterIndex: 0,
                                rangedLow: rangedLow,
                                rangedHigh: rangedHigh,
                                formulaLow: -1,
                                formulaHigh: -1,
                                formulaIndex: formulaIndex
                            }

                            $http({
                                method: "put",
                                url: "/api/devices/formula/" + device.port,
                                data: formulaCalcRequest
                            }).success(function (formulaCalcReturn) {
                                //console.log("port0.ACSensorConfiguration.Apply: Calculated formula value=" + JSON.stringify(formulaCalcReturn));
                                $scope.TempDevice.parameters[0].Formulas[formulaIndex].FormulaLowThreshold = formulaCalcReturn.formulaLow;
                                $scope.TempDevice.parameters[0].Formulas[formulaIndex].FormulaHighThreshold = formulaCalcReturn.formulaHigh;
                            });
                        }
                        //#endregion ACSensorConfiguration set functions

                        //#region ACSensorConfiguration apply
                        $scope.Apply = function () {
                            //console.log("port0.ACSensorConfiguration.Apply: Device to update=" + JSON.stringify($scope.TempDevice));
                            $scope.TempDevice.parameters[0].RangedLowThreshold = Number($scope.TempDevice.parameters[0].RangedLowThreshold).toFixed(2);
                            $scope.TempDevice.parameters[0].RangedHighThreshold = Number($scope.TempDevice.parameters[0].RangedHighThreshold).toFixed(2);

                            $http({
                                method: "put",
                                url: "/api/devices/" + device.port + "/0",// Default to parameter 0 
                                data: $scope.TempDevice
                            }).success(function (updatedDevice) {
                                //console.log("port0.ACSensorConfiguration.Apply: Updated Device=" + JSON.stringify(updatedDevice));
                                $scope.Device.DeviceModel = updatedDevice;
                                $modalInstance.close();
                            });
                        };
                        //#endregion ACSensorConfiguration apply

                        //#region ACSensorConfiguration cancel
                        $scope.Cancel = function () {
                            $modalInstance.dismiss('cancel');
                        };
                        //#endregion ACSensorConfiguration cancel
                    }
                });
            };
            //#endregion Configure device            

            //#region Socket messages
            socketManager.on('ACSensor', function (message) {
                if (message.Port == $scope.ThisPort) {
                    //console.log("Port" + $scope.ThisPort + ".Controller.SocketOn: ACSensor=" + JSON.stringify(message));

                    switch (message.Message) {
                        case "Value":
                            switch (message.Parameter.Id) {
                                case "AC":
                                    $scope.Device.DeviceModel.parameters[0] = message.Parameter;
                                    break;
                            }
                            break;

                        case "EventLog":
                            $scope.EventLog = message.Parameter;
                            break;
                    }
                }
            });
            //#endregion Socket messages
            //#endregion ACSensor device

        }
        //#endregion Controller: port 1
    ])

    .controller('port2', [
        '$scope',
        '$http',
        '$modal',
        'socketManager',
        //#region Controller: port2
        function ($scope, $http, $modal, socketManager) {
            
            // Controller Entry
            //#region Controller Entry
            $scope.$on('$stateChangeSuccess', function () {
                //console.log("Port" + $scope.ThisPort + ".Controller: Entry");
            });
            $scope.$on('$destroy', function (event) {
                socketManager.removeAllListeners();
            });
            //#endregion Controller Entry
            
            // Get Device
            //#region Get Device
            
            // Port and Device variables
            $scope.ThisPort = 2;
            $scope.Ports = [];
            $scope.Device = {};
            
            // Retreive array of Ports
            //  /api/ports
            $scope.GetPorts = function () {
                //console.log("Port" + $scope.ThisPort + ".Controller.GetPorts");
                $http({
                    method: "get",
                    url: "/api/ports"
                })
                .success(function (data) {
                        $scope.Ports = data;
                        $scope.Device = data[$scope.ThisPort];
                        //console.log("Port" + $scope.ThisPort + ".Controller.GetPorts: Device=" + JSON.stringify($scope.Device));
                    })
            }
            $scope.GetPorts();
            //#endregion Get Device
            
            // Get EventLog
            //#region EventLog
            // Retrieve array of Events
            //  /api/eventlog
            $scope.GetEventLog = function () {
                //console.log("Port" + $scope.ThisPort + ".Controller.GetEventLog");
                $http({
                    method: "get",
                    url: "/api/eventlog/" + $scope.ThisPort
                })
                .success(function (eventlog) {
                        $scope.EventLog = eventlog;
                        //console.log("Port" + $scope.ThisPort + ".Controller.GetEventLog: EventLog changed=" + JSON.stringify($scope.EventLog));
                    })
            }
            $scope.GetEventLog();
            //#endregion EventLog
            
            //===== NonDirectionalPower device =====
            //#region NonDirectionalPower device
            
            //#region NonDirectionalPower ChangeView
            $scope.ChangeFormulaIndex = function (option) {
                //console.log("port0.ChangeFormulaIndex: New formula=" + option);
                for (var i = 0; i < $scope.Device.DeviceModel.parameters.Power.Formulas.length; i++) {
                    if ($scope.Device.DeviceModel.parameters.Power.Formulas[i].Name == option) {
                        $scope.Device.DeviceModel.parameters.Power.FormulaIndex = i;
                        
                        // Persist the change
                        $http({
                            method: "put",
                            url: "/api/devices/" + $scope.Device.DeviceModel.port,
                            data: $scope.Device.DeviceModel
                        }).success(function (updatedmodel) {
                                //console.log("port0.ChangeFormulaIndex: Updated DeviceModel=" + JSON.stringify(updatedmodel));
                                device.DeviceModel = updatedmodel;
                                $modalInstance.close();
                            });
                    }
                }
            }
            //#endregion NonDirectionalPower ChangeView
            
            //#region Configure device
            $scope.ConfigureNonDirectionalPower = function (port, device) {
                //console.log("Inside ConfigureNonDirectionalPower for port=" + port)
                var modalInstance = $modal.open({
                    templateUrl: 'devices/NonDirectionalPowerConfigure.html',
                    controller: function ($scope, $modalInstance) {
                        
                        //#region ConfigureNonDirectionalPower preamble
                        $scope.modalTitle = "Configure Port " + port + ": " + device.Name;
                        
                        // Setup editing parameters
                        $scope.TempDevice = JSON.parse(JSON.stringify(device));
                        
                        $scope.NamePattern = /^[\w][\w ]{1,15}$/;
                        $scope.LowThresholdPattern = /^(([0-5]){0,1}|([0-4]{0,1}\.[0-9]{0,2})|(5\.[0]{0,2})){0,1}$/;
                        $scope.HighThresholdPattern = /^(([0-5]){0,1}|([0-4]{0,1}\.[0-9]{0,2})|(5\.[0]{0,2})){0,1}$/;
                        $scope.HoldTimeOptions = [{ option: '0' }, { option: '1' }, { option: '2' }, { option: '3' }, { option: '4' }, { option: '5' }];
                        $scope.QualifierOptions = [{ option: 'YES' }, { option: 'NO' }];
                        $scope.ModeOptions = [{ option: 'Normally Open' }, { option: 'Normally Closed' }];
                        //#endregion ConfigureNonDirectionalPower preamble
                        
                        //#region ConfigureNonDirectional set functions
                        $scope.SetPowerHoldTime = function (option) {
                            $scope.TempDevice.DeviceModel.parameters.Power.HoldTime = option;
                        }
                        
                        $scope.SetQualifier = function (option) {
                            $scope.TempDevice.DeviceModel.parameters.Power.Qualifier = (option === "YES") ? true:false;
                        }
                        
                        $scope.SetPTTHoldTime = function (option) {
                            $scope.TempDevice.DeviceModel.parameters.PTT.HoldTime = option;
                        }
                        
                        $scope.SetMode = function (option) {
                            $scope.TempDevice.DeviceModel.parameters.PTT.Mode = option;
                        }
                        
                        //#endregion ConfigureNonDirectional set functions
                        
                        //# region ConfigureNonDirectionalPower apply
                        $scope.Apply = function () {
                            //console.log("port0.ConfigureNonDirectionalPower.Apply: Passed DeviceModel=" + JSON.stringify($scope.TempDevice.DeviceModel));
                            
                            $http({
                                method: "put",
                                url: "/api/devices/" + port,
                                data: $scope.TempDevice.DeviceModel
                            }).success(function (updatedmodel) {
                                    //console.log("port0.ConfigureNonDirectionalPower.Apply: Updated DeviceModel=" + JSON.stringify(updatedmodel));
                                    device.DeviceModel = updatedmodel;
                                    $modalInstance.close();
                                });
                        };
                        //# endregion ConfigureNonDirectionalPower preamble
                        
                        //#region ConfigureNonDirectionalPower cancel
                        $scope.Cancel = function () {
                            $modalInstance.dismiss('cancel');
                        };
                        //#endregion ConfigureNonDirectionalPower cancel
                    }
                });
            };
            //#endregion Configure device            
            
            //#region Socket messages
            socketManager.on('NonDirectionalPower', function (message) {
                if (message.Port == $scope.ThisPort) {
                    //console.log("Port" + $scope.ThisPort + ".Controller.SocketOn: NonDirectionalPower=" + JSON.stringify(message));
                    
                    switch (message.Message) {
                        case "Value":
                            switch (message.Parameter.Id) {
                                case "Power":
                                    $scope.Device.DeviceModel.parameters.Power = message.Parameter;
                                    break;

                                case "Qualifier":
                                    $scope.Device.DeviceModel.parameters.PTT = message.Parameter;
                                    break;
                            }
                            break;

                        case "HoldTimeState":
                            if (Message.Parameter.Id == "Qualifier") {
                                $scope.Device.DeviceModel.parameters.PTT.HoldTimeState = message.Parameter.HoldTimeState;
                                $scope.Device.DeviceModel.parameters.PTT.Value = message.Parameter.Value;
                            }
                            break;

                        case "EventLog":
                            $scope.EventLog = message.Parameter;
                            break;
                    }
                }
            });
            //#endregion Socket messages

            //#endregion NonDirectionalPower device

            //===== DirectionalPower+VSWR device =====
            //#region DirectionalPower+VSWR device

            //#region Get Device Information
            // Retreive array of Ports
            //  /api/ports
            $scope.GetDeviceInfo = function () {
                //console.log("Port" + $scope.ThisPort + ".Controller.GetDeviceInfo");
                $http({
                    method: "get",
                    url: "/api/devices/info/" + $scope.ThisPort
                })
                    .success(function (deviceInfo) {
                        $scope.Device.DeviceModel.device = deviceInfo;
                        //console.log("Port" + $scope.ThisPort + ".Controller.GetDeviceInfo: DeviceInfo=" + JSON.stringify(deviceInfo));
                    })
            }
            $scope.GetDeviceInfo();
            //#endregion Get Device Information

            //#region Formula Changes
            //#region ForwardPower Formula Change
            $scope.ChangeForwardPowerFormulaIndex = function (option) {
                //console.log("port" + $scope.ThisPort + ".ChangeForwardPowerFormulaIndex: New formula=" + option);
                for (var i = 0; i < $scope.Device.DeviceModel.parameters.ForwardPower.Formulas.length; i++) {
                    if ($scope.Device.DeviceModel.parameters.ForwardPower.Formulas[i].Name == option) {
                        $scope.Device.DeviceModel.parameters.ForwardPower.FormulaIndex = i;

                        // Persist the change
                        $http({
                            method: "put",
                            url: "/api/devices/" + $scope.Device.DeviceModel.port,
                            data: $scope.Device.DeviceModel
                        }).success(function (updatedmodel) {
                            //console.log("port" + $scope.ThisPort + ".ChangeForwardPowerFormulaIndex: Updated DeviceModel=" + JSON.stringify(updatedmodel));
                            device.DeviceModel = updatedmodel;
                            $modalInstance.close();
                        });
                    }
                }
            }
            //#endregion ForwardPower Formula Change

            //#region ReflectedPower Formula Change
            $scope.ChangeReflectedPowerFormulaIndex = function (option) {
                //console.log("port" + $scope.ThisPort + ".ChangeReflectedPowerFormulaIndex: New formula=" + option);
                for (var i = 0; i < $scope.Device.DeviceModel.parameters.ReflectedPower.Formulas.length; i++) {
                    if ($scope.Device.DeviceModel.parameters.ReflectedPower.Formulas[i].Name == option) {
                        $scope.Device.DeviceModel.parameters.ReflectedPower.FormulaIndex = i;

                        // Persist the change
                        $http({
                            method: "put",
                            url: "/api/devices/" + $scope.Device.DeviceModel.port,
                            data: $scope.Device.DeviceModel
                        }).success(function (updatedmodel) {
                            //console.log("port" + $scope.ThisPort + ".ChangeReflectedPowerFormulaIndex: Updated DeviceModel=" + JSON.stringify(updatedmodel));
                            device.DeviceModel = updatedmodel;
                            $modalInstance.close();
                        });
                    }
                }
            }
            //#endregion ReflectedPower Formula Change

            //#region Temperature Formula Change
            $scope.ChangeTemperatureFormulaIndex = function (option) {
                //console.log("port" + $scope.ThisPort + ".ChangeTemperatureFormulaIndex: New formula=" + option);
                for (var i = 0; i < $scope.Device.DeviceModel.parameters.Temperature.Formulas.length; i++) {
                    if ($scope.Device.DeviceModel.parameters.Temperature.Formulas[i].Name == option) {
                        $scope.Device.DeviceModel.parameters.Temperature.FormulaIndex = i;

                        // Persist the change
                        $http({
                            method: "put",
                            url: "/api/devices/" + $scope.Device.DeviceModel.port,
                            data: $scope.Device.DeviceModel
                        }).success(function (updatedmodel) {
                            //console.log("port" + $scope.ThisPort + ".ChangeTemperatureFormulaIndex: Updated DeviceModel=" + JSON.stringify(updatedmodel));
                            device.DeviceModel = updatedmodel;
                            $modalInstance.close();
                        });
                    }
                }
            }
            //#endregion Temperature Formula Change
            //#endregion Formula Changes


            //#region Socket messages
            socketManager.on('DirectionalPower+VSWR', function (message) {
                if (message.Port == $scope.ThisPort) {
                    //console.log("Port" + $scope.ThisPort + ".Controller.SocketOn: DirectionalPower+VSWR=" + JSON.stringify(message));

                    switch (message.Message) {
                        case "DeviceInformation":
                            $scope.Device.DeviceModel.device = message.device;
                            break;

                        case "ForwardPower":
                            $scope.Device.DeviceModel.parameters.ForwardPower = message.Parameter.ForwardPower;
                            break;

                        case "ReflectedPower":
                            $scope.Device.DeviceModel.parameters.ReflectedPower = message.Parameter.ReflectedPower;
                            break;

                        case "VSWR":
                            $scope.Device.DeviceModel.parameters.VSWR = message.Parameter.VSWR;
                            break;

                        case "Temperature":
                            $scope.Device.DeviceModel.parameters.Temperature = message.Parameter.Temperature;
                            break;
                    }
                }
            });
            //#endregion Socket messages
            //#endregion DirectionalPower+VSWR device

            //===== TempHumidity sensor =====
            //#region TempHumiditySensor device
            //#region Configure device
            $scope.TempHumiditySensorConfiguration = function (device, parameterIndex) {
                //console.log("Inside TempHumiditySensorConfiguration for port=" + device.port)
                var modalInstance = $modal.open({
                    templateUrl: 'devices/TempHumiditySensorConfiguration.html',
                    scope: $scope,
                    controller: function ($scope, $modalInstance) {

                        //#region TempHumiditySensorConfiguration preamble
                        $scope.modalTitle = "Configure Port " + device.port + ": " + device.title;
                        $scope.parameterIndex = parameterIndex;
                        $scope.TempDevice = JSON.parse(JSON.stringify(device));
                        $scope.RangedLowThresholdPattern = /^([0-5]{1}|[0-4]?\.[0-9]{0,2}|5\.[0]{0,2})$/;
                        $scope.RangedHighThresholdPattern = /^([0-5]{1}|[0-4]?\.[0-9]{0,2}|5\.[0]{0,2})$/;
                        $scope.HoldTimeOptions = [{ option: '0' }, { option: '1' }, { option: '2' }, { option: '3' }, { option: '4' }, { option: '5' }];
                        //#endregion TempHumiditySensorConfiguration preamble

                        //#region TempHumiditySensorConfiguration set functions
                        $scope.SetHoldTime = function (option) {
                            $scope.TempDevice.parameters[$scope.parameterIndex].HoldTime = Number(option);
                        }

                        $scope.SelectFormula = function (option) {
                            $scope.TempDevice.parameters[$scope.parameterIndex].FormulaIndex = option;
                        }

                        $scope.Recalculate = function (rangedLow, rangedHigh, formulaIndex) {
                            var formulaCalcRequest = {
                                parmeterIndex: $scope.parameterIndex,
                                rangedLow: rangedLow,
                                rangedHigh: rangedHigh,
                                formulaLow: -1,
                                formulaHigh: -1,
                                formulaIndex: formulaIndex
                            }

                            $http({
                                method: "put",
                                url: "/api/devices/formula/" + device.port,
                                data: formulaCalcRequest
                            }).success(function (formulaCalcReturn) {
                                //console.log("port0.TempHumiditySensorConfiguration.Apply: Calculated formula value=" + JSON.stringify(formulaCalcReturn));
                                $scope.TempDevice.parameters[$scope.parameterIndex].Formulas[formulaIndex].FormulaLowThreshold = formulaCalcReturn.formulaLow;
                                $scope.TempDevice.parameters[$scope.parameterIndex].Formulas[formulaIndex].FormulaHighThreshold = formulaCalcReturn.formulaHigh;
                            });
                        }
                        //#endregion TempHumiditySensorConfiguration set functions

                        //#region TempHumiditySensorConfiguration apply
                        $scope.Apply = function () {
                            //console.log("port0.TempHumiditySensorConfiguration.Apply: Device to update=" + JSON.stringify($scope.TempDevice));
                            $scope.TempDevice.parameters[$scope.parameterIndex].RangedLowThreshold = Number($scope.TempDevice.parameters[$scope.parameterIndex].RangedLowThreshold).toFixed(2);
                            $scope.TempDevice.parameters[$scope.parameterIndex].RangedHighThreshold = Number($scope.TempDevice.parameters[$scope.parameterIndex].RangedHighThreshold).toFixed(2);

                            $http({
                                method: "put",
                                url: "/api/devices/" + device.port + "/" + $scope.parameterIndex,
                                data: $scope.TempDevice
                            }).success(function (updatedDevice) {
                                //console.log("port0.TempHumiditySensorConfiguration.Apply: Updated Device=" + JSON.stringify(updatedDevice));
                                $scope.Device.DeviceModel = updatedDevice;
                                $modalInstance.close();
                            });
                        };
                        //#endregion TempHumiditySensorConfiguration apply

                        //#region TempHumiditySensorConfiguration cancel
                        $scope.Cancel = function () {
                            $modalInstance.dismiss('cancel');
                        };
                        //#endregion TempHumiditySensorConfiguration cancel
                    }
                });
            };
            //#endregion Configure device            

            //#region Socket messages
            socketManager.on('TempHumiditySensor', function (message) {
                if (message.Port == $scope.ThisPort) {
                    //console.log("Port" + $scope.ThisPort + ".Controller.SocketOn: TempHumiditySensor=" + JSON.stringify(message));

                    switch (message.Message) {
                        case "Value":
                            switch (message.Parameter.Id) {
                                case "Temperature":
                                    $scope.Device.DeviceModel.parameters[1] = message.Parameter;
                                    break;

                                case "Humidity":
                                    $scope.Device.DeviceModel.parameters[0] = message.Parameter;
                                    break;
                            }
                            break;

                        case "EventLog":
                            $scope.EventLog = message.Parameter;
                            break;
                    }
                }
            });
            //#endregion Socket messages
            //#endregion TempHumiditySensor device

            //===== DoorSensor device =====
            //#region DoorSensor device
            //#region Configure device
            $scope.DoorSensorConfiguration = function (device) {
                //console.log("Inside DoorSensorConfiguration for port=" + device.port)
                var modalInstance = $modal.open({
                    templateUrl: 'devices/DoorSensorConfiguration.html',
                    scope: $scope,
                    controller: function ($scope, $modalInstance) {

                        //#region DoorSensorConfiguration preamble
                        $scope.modalTitle = "Configure Port " + device.port + ": " + device.title;
                        $scope.TempDevice = JSON.parse(JSON.stringify(device));
                        $scope.ModeOptions = [{ option: 'Normally Open' }, { option: 'Normally Closed' }];
                        $scope.HoldTimeOptions = [{ option: '0' }, { option: '1' }, { option: '2' }, { option: '3' }, { option: '4' }, { option: '5' }];
                        //#endregion DoorSensorConfiguration preamble

                        //#region DoorSensorConfiguration set functions
                        $scope.SetHoldTime = function (option) {
                            $scope.TempDevice.parameters[0].HoldTime = Number(option);
                        }
                        $scope.SetMode = function (option) {
                            $scope.TempDevice.parameters[0].Mode = option;
                        }
                        //#endregion DoorSensorConfiguration set functions

                        //#region DoorSensorConfiguration apply
                        $scope.Apply = function () {
                            $http({
                                method: "put",
                                url: "/api/devices/" + device.port + "/0",// Just one parameter to worry about
                                data: $scope.TempDevice
                            }).success(function (updatedDevice) {
                                //console.log("port0.DoorSensorConfiguration.Apply: Updated Device=" + JSON.stringify(updatedDevice));
                                $scope.Device.DeviceModel = updatedDevice;
                                $modalInstance.close();
                            });
                        };
                        //#endregion DoorSensorConfiguration apply

                        //#region DoorSensorConfiguration cancel
                        $scope.Cancel = function () {
                            $modalInstance.dismiss('cancel');
                        };
                        //#endregion DoorSensorConfiguration cancel
                    }
                });
            };
            //#endregion Configure device            

            //#region Socket messages
            socketManager.on('DoorSensor', function (message) {
                if (message.Port == $scope.ThisPort) {
                    switch (message.Message) {
                        case "Value":
                            switch (message.Parameter.Id) {
                                case "Door":
                                    //console.log("Port" + $scope.ThisPort + ".Controller.SocketOn: DoorSensor=" + JSON.stringify(message));
                                    $scope.Device.DeviceModel.parameters[0] = message.Parameter;
                                    break;
                            }
                            break;

                        case "EventLog":
                            $scope.EventLog = message.Parameter;
                            break;
                    }
                }
            });
            //#endregion Socket messages
            //#endregion DoorSensor device

            //===== ACSensor device =====
            //#region ACSensor device
            //#region Configure device
            $scope.ACSensorConfiguration = function (device) {
                //console.log("Inside ACSensorConfiguration for port=" + device.port)
                var modalInstance = $modal.open({
                    templateUrl: 'devices/ACSensorConfiguration.html',
                    scope: $scope,
                    controller: function ($scope, $modalInstance) {

                        //#region ACSensorConfiguration preamble
                        $scope.modalTitle = "Configure Port " + device.port + ": " + device.title;
                        $scope.TempDevice = JSON.parse(JSON.stringify(device));
                        $scope.RangedLowThresholdPattern = /^([0-5]{1}|[0-4]?\.[0-9]{0,2}|5\.[0]{0,2})$/;
                        $scope.RangedHighThresholdPattern = /^([0-5]{1}|[0-4]?\.[0-9]{0,2}|5\.[0]{0,2})$/;
                        $scope.HoldTimeOptions = [{ option: '0' }, { option: '1' }, { option: '2' }, { option: '3' }, { option: '4' }, { option: '5' }];
                        //#endregion ACSensorConfiguration preamble

                        //#region ACSensorConfiguration set functions
                        $scope.SetHoldTime = function (option) {
                            $scope.TempDevice.parameters[0].HoldTime = Number(option);
                        }

                        $scope.SelectFormula = function (option) {
                            $scope.TempDevice.parameters[0].FormulaIndex = option;
                        }

                        $scope.Recalculate = function (rangedLow, rangedHigh, formulaIndex) {
                            var formulaCalcRequest = {
                                parmeterIndex: 0,
                                rangedLow: rangedLow,
                                rangedHigh: rangedHigh,
                                formulaLow: -1,
                                formulaHigh: -1,
                                formulaIndex: formulaIndex
                            }

                            $http({
                                method: "put",
                                url: "/api/devices/formula/" + device.port,
                                data: formulaCalcRequest
                            }).success(function (formulaCalcReturn) {
                                //console.log("port0.ACSensorConfiguration.Apply: Calculated formula value=" + JSON.stringify(formulaCalcReturn));
                                $scope.TempDevice.parameters[0].Formulas[formulaIndex].FormulaLowThreshold = formulaCalcReturn.formulaLow;
                                $scope.TempDevice.parameters[0].Formulas[formulaIndex].FormulaHighThreshold = formulaCalcReturn.formulaHigh;
                            });
                        }
                        //#endregion ACSensorConfiguration set functions

                        //#region ACSensorConfiguration apply
                        $scope.Apply = function () {
                            //console.log("port0.ACSensorConfiguration.Apply: Device to update=" + JSON.stringify($scope.TempDevice));
                            $scope.TempDevice.parameters[0].RangedLowThreshold = Number($scope.TempDevice.parameters[0].RangedLowThreshold).toFixed(2);
                            $scope.TempDevice.parameters[0].RangedHighThreshold = Number($scope.TempDevice.parameters[0].RangedHighThreshold).toFixed(2);

                            $http({
                                method: "put",
                                url: "/api/devices/" + device.port + "/0",// Default to parameter 0 
                                data: $scope.TempDevice
                            }).success(function (updatedDevice) {
                                //console.log("port0.ACSensorConfiguration.Apply: Updated Device=" + JSON.stringify(updatedDevice));
                                $scope.Device.DeviceModel = updatedDevice;
                                $modalInstance.close();
                            });
                        };
                        //#endregion ACSensorConfiguration apply

                        //#region ACSensorConfiguration cancel
                        $scope.Cancel = function () {
                            $modalInstance.dismiss('cancel');
                        };
                        //#endregion ACSensorConfiguration cancel
                    }
                });
            };
            //#endregion Configure device            

            //#region Socket messages
            socketManager.on('ACSensor', function (message) {
                if (message.Port == $scope.ThisPort) {
                    //console.log("Port" + $scope.ThisPort + ".Controller.SocketOn: ACSensor=" + JSON.stringify(message));

                    switch (message.Message) {
                        case "Value":
                            switch (message.Parameter.Id) {
                                case "AC":
                                    $scope.Device.DeviceModel.parameters[0] = message.Parameter;
                                    break;
                            }
                            break;

                        case "EventLog":
                            $scope.EventLog = message.Parameter;
                            break;
                    }
                }
            });
            //#endregion Socket messages
            //#endregion ACSensor device

        }
        //#endregion Controller: port2
    ])

    .controller('port3', [
        '$scope',
        '$http',
        '$modal',
        'socketManager',
        //#region Controller: port3
        function ($scope, $http, $modal, socketManager) {
            
            // Controller Entry
            //#region Controller Entry
            $scope.$on('$stateChangeSuccess', function () {
                //console.log("Port" + $scope.ThisPort + ".Controller: Entry");
            });
            $scope.$on('$destroy', function (event) {
                socketManager.removeAllListeners();
            });
            //#endregion Controller Entry
            
            // Get Device
            //#region Get Device
            
            // Port and Device variables
            $scope.ThisPort = 3;
            $scope.Ports = [];
            $scope.Device = {};
            
            // Retreive array of Ports
            //  /api/ports
            $scope.GetPorts = function () {
                //console.log("Port" + $scope.ThisPort + ".Controller.GetPorts");
                $http({
                    method: "get",
                    url: "/api/ports"
                })
                .success(function (data) {
                        $scope.Ports = data;
                        $scope.Device = data[$scope.ThisPort];
                        //console.log("Port" + $scope.ThisPort + ".Controller.GetPorts: Device=" + JSON.stringify($scope.Device));
                    })
            }
            $scope.GetPorts();
            //#endregion Get Device
            
            // Get EventLog
            //#region EventLog
            // Retrieve array of Events
            //  /api/eventlog
            $scope.GetEventLog = function () {
                //console.log("Port" + $scope.ThisPort + ".Controller.GetEventLog");
                $http({
                    method: "get",
                    url: "/api/eventlog/" + $scope.ThisPort
                })
                .success(function (eventlog) {
                        $scope.EventLog = eventlog;
                        //console.log("Port" + $scope.ThisPort + ".Controller.GetEventLog: EventLog changed=" + JSON.stringify($scope.EventLog));
                    })
            }
            $scope.GetEventLog();
            //#endregion EventLog
            
            //===== NonDirectionalPower device =====
            //#region NonDirectionalPower device
            
            //#region NonDirectionalPower ChangeView
            $scope.ChangeFormulaIndex = function (option) {
                //console.log("port0.ChangeFormulaIndex: New formula=" + option);
                for (var i = 0; i < $scope.Device.DeviceModel.parameters.Power.Formulas.length; i++) {
                    if ($scope.Device.DeviceModel.parameters.Power.Formulas[i].Name == option) {
                        $scope.Device.DeviceModel.parameters.Power.FormulaIndex = i;
                        
                        // Persist the change
                        $http({
                            method: "put",
                            url: "/api/devices/" + $scope.Device.DeviceModel.port,
                            data: $scope.Device.DeviceModel
                        }).success(function (updatedmodel) {
                                //console.log("port0.ChangeFormulaIndex: Updated DeviceModel=" + JSON.stringify(updatedmodel));
                                device.DeviceModel = updatedmodel;
                                $modalInstance.close();
                            });
                    }
                }
            }
            //#endregion NonDirectionalPower ChangeView
            
            //#region Configure device
            $scope.ConfigureNonDirectionalPower = function (port, device) {
                //console.log("Inside ConfigureNonDirectionalPower for port=" + port)
                var modalInstance = $modal.open({
                    templateUrl: 'devices/NonDirectionalPowerConfigure.html',
                    controller: function ($scope, $modalInstance) {
                        
                        //#region ConfigureNonDirectionalPower preamble
                        $scope.modalTitle = "Configure Port " + port + ": " + device.Name;
                        
                        // Setup editing parameters
                        $scope.TempDevice = JSON.parse(JSON.stringify(device));
                        
                        $scope.NamePattern = /^[\w][\w ]{1,15}$/;
                        $scope.LowThresholdPattern = /^(([0-5]){0,1}|([0-4]{0,1}\.[0-9]{0,2})|(5\.[0]{0,2})){0,1}$/;
                        $scope.HighThresholdPattern = /^(([0-5]){0,1}|([0-4]{0,1}\.[0-9]{0,2})|(5\.[0]{0,2})){0,1}$/;
                        $scope.HoldTimeOptions = [{ option: '0' }, { option: '1' }, { option: '2' }, { option: '3' }, { option: '4' }, { option: '5' }];
                        $scope.QualifierOptions = [{ option: 'YES' }, { option: 'NO' }];
                        $scope.ModeOptions = [{ option: 'Normally Open' }, { option: 'Normally Closed' }];
                        //#endregion ConfigureNonDirectionalPower preamble
                        
                        //#region ConfigureNonDirectional set functions
                        $scope.SetPowerHoldTime = function (option) {
                            $scope.TempDevice.DeviceModel.parameters.Power.HoldTime = option;
                        }
                        
                        $scope.SetQualifier = function (option) {
                            $scope.TempDevice.DeviceModel.parameters.Power.Qualifier = (option === "YES") ? true:false;
                        }
                        
                        $scope.SetPTTHoldTime = function (option) {
                            $scope.TempDevice.DeviceModel.parameters.PTT.HoldTime = option;
                        }
                        
                        $scope.SetMode = function (option) {
                            $scope.TempDevice.DeviceModel.parameters.PTT.Mode = option;
                        }
                        
                        //#endregion ConfigureNonDirectional set functions
                        
                        //# region ConfigureNonDirectionalPower apply
                        $scope.Apply = function () {
                            //console.log("port0.ConfigureNonDirectionalPower.Apply: Passed DeviceModel=" + JSON.stringify($scope.TempDevice.DeviceModel));
                            
                            $http({
                                method: "put",
                                url: "/api/devices/" + port,
                                data: $scope.TempDevice.DeviceModel
                            }).success(function (updatedmodel) {
                                    //console.log("port0.ConfigureNonDirectionalPower.Apply: Updated DeviceModel=" + JSON.stringify(updatedmodel));
                                    device.DeviceModel = updatedmodel;
                                    $modalInstance.close();
                                });
                        };
                        //# endregion ConfigureNonDirectionalPower preamble
                        
                        //#region ConfigureNonDirectionalPower cancel
                        $scope.Cancel = function () {
                            $modalInstance.dismiss('cancel');
                        };
                        //#endregion ConfigureNonDirectionalPower cancel
                    }
                });
            };
            //#endregion Configure device            
            
            //#region Socket messages
            socketManager.on('NonDirectionalPower', function (message) {
                if (message.Port == $scope.ThisPort) {
                    //console.log("Port" + $scope.ThisPort + ".Controller.SocketOn: NonDirectionalPower=" + JSON.stringify(message));
                    
                    switch (message.Message) {
                        case "Value":
                            switch (message.Parameter.Id) {
                                case "Power":
                                    $scope.Device.DeviceModel.parameters.Power = message.Parameter;
                                    break;

                                case "Qualifier":
                                    $scope.Device.DeviceModel.parameters.PTT = message.Parameter;
                                    break;
                            }
                            break;

                        case "HoldTimeState":
                            if (Message.Parameter.Id == "Qualifier") {
                                $scope.Device.DeviceModel.parameters.PTT.HoldTimeState = message.Parameter.HoldTimeState;
                                $scope.Device.DeviceModel.parameters.PTT.Value = message.Parameter.Value;
                            }
                            break;

                        case "EventLog":
                            $scope.EventLog = message.Parameter;
                            break;
                    }
                }
            });
            //#endregion Socket messages

            //#endregion NonDirectionalPower device

            //===== DirectionalPower+VSWR device =====
            //#region DirectionalPower+VSWR device

            //#region Get Device Information
            // Retreive array of Ports
            //  /api/ports
            $scope.GetDeviceInfo = function () {
                //console.log("Port" + $scope.ThisPort + ".Controller.GetDeviceInfo");
                $http({
                    method: "get",
                    url: "/api/devices/info/" + $scope.ThisPort
                })
                    .success(function (deviceInfo) {
                        $scope.Device.DeviceModel.device = deviceInfo;
                        //console.log("Port" + $scope.ThisPort + ".Controller.GetDeviceInfo: DeviceInfo=" + JSON.stringify(deviceInfo));
                    })
            }
            $scope.GetDeviceInfo();
            //#endregion Get Device Information

            //#region Formula Changes
            //#region ForwardPower Formula Change
            $scope.ChangeForwardPowerFormulaIndex = function (option) {
                //console.log("port" + $scope.ThisPort + ".ChangeForwardPowerFormulaIndex: New formula=" + option);
                for (var i = 0; i < $scope.Device.DeviceModel.parameters.ForwardPower.Formulas.length; i++) {
                    if ($scope.Device.DeviceModel.parameters.ForwardPower.Formulas[i].Name == option) {
                        $scope.Device.DeviceModel.parameters.ForwardPower.FormulaIndex = i;

                        // Persist the change
                        $http({
                            method: "put",
                            url: "/api/devices/" + $scope.Device.DeviceModel.port,
                            data: $scope.Device.DeviceModel
                        }).success(function (updatedmodel) {
                            //console.log("port" + $scope.ThisPort + ".ChangeForwardPowerFormulaIndex: Updated DeviceModel=" + JSON.stringify(updatedmodel));
                            device.DeviceModel = updatedmodel;
                            $modalInstance.close();
                        });
                    }
                }
            }
            //#endregion ForwardPower Formula Change

            //#region ReflectedPower Formula Change
            $scope.ChangeReflectedPowerFormulaIndex = function (option) {
                //console.log("port" + $scope.ThisPort + ".ChangeReflectedPowerFormulaIndex: New formula=" + option);
                for (var i = 0; i < $scope.Device.DeviceModel.parameters.ReflectedPower.Formulas.length; i++) {
                    if ($scope.Device.DeviceModel.parameters.ReflectedPower.Formulas[i].Name == option) {
                        $scope.Device.DeviceModel.parameters.ReflectedPower.FormulaIndex = i;

                        // Persist the change
                        $http({
                            method: "put",
                            url: "/api/devices/" + $scope.Device.DeviceModel.port,
                            data: $scope.Device.DeviceModel
                        }).success(function (updatedmodel) {
                            //console.log("port" + $scope.ThisPort + ".ChangeReflectedPowerFormulaIndex: Updated DeviceModel=" + JSON.stringify(updatedmodel));
                            device.DeviceModel = updatedmodel;
                            $modalInstance.close();
                        });
                    }
                }
            }
            //#endregion ReflectedPower Formula Change

            //#region Temperature Formula Change
            $scope.ChangeTemperatureFormulaIndex = function (option) {
                //console.log("port" + $scope.ThisPort + ".ChangeTemperatureFormulaIndex: New formula=" + option);
                for (var i = 0; i < $scope.Device.DeviceModel.parameters.Temperature.Formulas.length; i++) {
                    if ($scope.Device.DeviceModel.parameters.Temperature.Formulas[i].Name == option) {
                        $scope.Device.DeviceModel.parameters.Temperature.FormulaIndex = i;

                        // Persist the change
                        $http({
                            method: "put",
                            url: "/api/devices/" + $scope.Device.DeviceModel.port,
                            data: $scope.Device.DeviceModel
                        }).success(function (updatedmodel) {
                            //console.log("port" + $scope.ThisPort + ".ChangeTemperatureFormulaIndex: Updated DeviceModel=" + JSON.stringify(updatedmodel));
                            device.DeviceModel = updatedmodel;
                            $modalInstance.close();
                        });
                    }
                }
            }
            //#endregion Temperature Formula Change
            //#endregion Formula Changes


            //#region Socket messages
            socketManager.on('DirectionalPower+VSWR', function (message) {
                if (message.Port == $scope.ThisPort) {
                    //console.log("Port" + $scope.ThisPort + ".Controller.SocketOn: DirectionalPower+VSWR=" + JSON.stringify(message));

                    switch (message.Message) {
                        case "DeviceInformation":
                            $scope.Device.DeviceModel.device = message.device;
                            break;

                        case "ForwardPower":
                            $scope.Device.DeviceModel.parameters.ForwardPower = message.Parameter.ForwardPower;
                            break;

                        case "ReflectedPower":
                            $scope.Device.DeviceModel.parameters.ReflectedPower = message.Parameter.ReflectedPower;
                            break;

                        case "VSWR":
                            $scope.Device.DeviceModel.parameters.VSWR = message.Parameter.VSWR;
                            break;

                        case "Temperature":
                            $scope.Device.DeviceModel.parameters.Temperature = message.Parameter.Temperature;
                            break;
                    }
                }
            });
            //#endregion Socket messages
            //#endregion DirectionalPower+VSWR device

            //===== TempHumidity sensor =====
            //#region TempHumiditySensor device
            //#region Configure device
            $scope.TempHumiditySensorConfiguration = function (device, parameterIndex) {
                //console.log("Inside TempHumiditySensorConfiguration for port=" + device.port)
                var modalInstance = $modal.open({
                    templateUrl: 'devices/TempHumiditySensorConfiguration.html',
                    scope: $scope,
                    controller: function ($scope, $modalInstance) {

                        //#region TempHumiditySensorConfiguration preamble
                        $scope.modalTitle = "Configure Port " + device.port + ": " + device.title;
                        $scope.parameterIndex = parameterIndex;
                        $scope.TempDevice = JSON.parse(JSON.stringify(device));
                        $scope.RangedLowThresholdPattern = /^([0-5]{1}|[0-4]?\.[0-9]{0,2}|5\.[0]{0,2})$/;
                        $scope.RangedHighThresholdPattern = /^([0-5]{1}|[0-4]?\.[0-9]{0,2}|5\.[0]{0,2})$/;
                        $scope.HoldTimeOptions = [{ option: '0' }, { option: '1' }, { option: '2' }, { option: '3' }, { option: '4' }, { option: '5' }];
                        //#endregion TempHumiditySensorConfiguration preamble

                        //#region TempHumiditySensorConfiguration set functions
                        $scope.SetHoldTime = function (option) {
                            $scope.TempDevice.parameters[$scope.parameterIndex].HoldTime = Number(option);
                        }

                        $scope.SelectFormula = function (option) {
                            $scope.TempDevice.parameters[$scope.parameterIndex].FormulaIndex = option;
                        }

                        $scope.Recalculate = function (rangedLow, rangedHigh, formulaIndex) {
                            var formulaCalcRequest = {
                                parmeterIndex: $scope.parameterIndex,
                                rangedLow: rangedLow,
                                rangedHigh: rangedHigh,
                                formulaLow: -1,
                                formulaHigh: -1,
                                formulaIndex: formulaIndex
                            }

                            $http({
                                method: "put",
                                url: "/api/devices/formula/" + device.port,
                                data: formulaCalcRequest
                            }).success(function (formulaCalcReturn) {
                                //console.log("port0.TempHumiditySensorConfiguration.Apply: Calculated formula value=" + JSON.stringify(formulaCalcReturn));
                                $scope.TempDevice.parameters[$scope.parameterIndex].Formulas[formulaIndex].FormulaLowThreshold = formulaCalcReturn.formulaLow;
                                $scope.TempDevice.parameters[$scope.parameterIndex].Formulas[formulaIndex].FormulaHighThreshold = formulaCalcReturn.formulaHigh;
                            });
                        }
                        //#endregion TempHumiditySensorConfiguration set functions

                        //#region TempHumiditySensorConfiguration apply
                        $scope.Apply = function () {
                            //console.log("port0.TempHumiditySensorConfiguration.Apply: Device to update=" + JSON.stringify($scope.TempDevice));
                            $scope.TempDevice.parameters[$scope.parameterIndex].RangedLowThreshold = Number($scope.TempDevice.parameters[$scope.parameterIndex].RangedLowThreshold).toFixed(2);
                            $scope.TempDevice.parameters[$scope.parameterIndex].RangedHighThreshold = Number($scope.TempDevice.parameters[$scope.parameterIndex].RangedHighThreshold).toFixed(2);

                            $http({
                                method: "put",
                                url: "/api/devices/" + device.port + "/" + $scope.parameterIndex,
                                data: $scope.TempDevice
                            }).success(function (updatedDevice) {
                                //console.log("port0.TempHumiditySensorConfiguration.Apply: Updated Device=" + JSON.stringify(updatedDevice));
                                $scope.Device.DeviceModel = updatedDevice;
                                $modalInstance.close();
                            });
                        };
                        //#endregion TempHumiditySensorConfiguration apply

                        //#region TempHumiditySensorConfiguration cancel
                        $scope.Cancel = function () {
                            $modalInstance.dismiss('cancel');
                        };
                        //#endregion TempHumiditySensorConfiguration cancel
                    }
                });
            };
            //#endregion Configure device            

            //#region Socket messages
            socketManager.on('TempHumiditySensor', function (message) {
                if (message.Port == $scope.ThisPort) {
                    //console.log("Port" + $scope.ThisPort + ".Controller.SocketOn: TempHumiditySensor=" + JSON.stringify(message));

                    switch (message.Message) {
                        case "Value":
                            switch (message.Parameter.Id) {
                                case "Temperature":
                                    $scope.Device.DeviceModel.parameters[1] = message.Parameter;
                                    break;

                                case "Humidity":
                                    $scope.Device.DeviceModel.parameters[0] = message.Parameter;
                                    break;
                            }
                            break;

                        case "EventLog":
                            $scope.EventLog = message.Parameter;
                            break;
                    }
                }
            });
            //#endregion Socket messages
            //#endregion TempHumiditySensor device

            //===== DoorSensor device =====
            //#region DoorSensor device
            //#region Configure device
            $scope.DoorSensorConfiguration = function (device) {
                //console.log("Inside DoorSensorConfiguration for port=" + device.port)
                var modalInstance = $modal.open({
                    templateUrl: 'devices/DoorSensorConfiguration.html',
                    scope: $scope,
                    controller: function ($scope, $modalInstance) {

                        //#region DoorSensorConfiguration preamble
                        $scope.modalTitle = "Configure Port " + device.port + ": " + device.title;
                        $scope.TempDevice = JSON.parse(JSON.stringify(device));
                        $scope.ModeOptions = [{ option: 'Normally Open' }, { option: 'Normally Closed' }];
                        $scope.HoldTimeOptions = [{ option: '0' }, { option: '1' }, { option: '2' }, { option: '3' }, { option: '4' }, { option: '5' }];
                        //#endregion DoorSensorConfiguration preamble

                        //#region DoorSensorConfiguration set functions
                        $scope.SetHoldTime = function (option) {
                            $scope.TempDevice.parameters[0].HoldTime = Number(option);
                        }
                        $scope.SetMode = function (option) {
                            $scope.TempDevice.parameters[0].Mode = option;
                        }
                        //#endregion DoorSensorConfiguration set functions

                        //#region DoorSensorConfiguration apply
                        $scope.Apply = function () {
                            $http({
                                method: "put",
                                url: "/api/devices/" + device.port + "/0",// Just one parameter to worry about
                                data: $scope.TempDevice
                            }).success(function (updatedDevice) {
                                //console.log("port0.DoorSensorConfiguration.Apply: Updated Device=" + JSON.stringify(updatedDevice));
                                $scope.Device.DeviceModel = updatedDevice;
                                $modalInstance.close();
                            });
                        };
                        //#endregion DoorSensorConfiguration apply

                        //#region DoorSensorConfiguration cancel
                        $scope.Cancel = function () {
                            $modalInstance.dismiss('cancel');
                        };
                        //#endregion DoorSensorConfiguration cancel
                    }
                });
            };
            //#endregion Configure device            

            //#region Socket messages
            socketManager.on('DoorSensor', function (message) {
                if (message.Port == $scope.ThisPort) {
                    switch (message.Message) {
                        case "Value":
                            switch (message.Parameter.Id) {
                                case "Door":
                                    //console.log("Port" + $scope.ThisPort + ".Controller.SocketOn: DoorSensor=" + JSON.stringify(message));
                                    $scope.Device.DeviceModel.parameters[0] = message.Parameter;
                                    break;
                            }
                            break;

                        case "EventLog":
                            $scope.EventLog = message.Parameter;
                            break;
                    }
                }
            });
            //#endregion Socket messages
            //#endregion DoorSensor device

            //===== ACSensor device =====
            //#region ACSensor device
            //#region Configure device
            $scope.ACSensorConfiguration = function (device) {
                //console.log("Inside ACSensorConfiguration for port=" + device.port)
                var modalInstance = $modal.open({
                    templateUrl: 'devices/ACSensorConfiguration.html',
                    scope: $scope,
                    controller: function ($scope, $modalInstance) {

                        //#region ACSensorConfiguration preamble
                        $scope.modalTitle = "Configure Port " + device.port + ": " + device.title;
                        $scope.TempDevice = JSON.parse(JSON.stringify(device));
                        $scope.RangedLowThresholdPattern = /^([0-5]{1}|[0-4]?\.[0-9]{0,2}|5\.[0]{0,2})$/;
                        $scope.RangedHighThresholdPattern = /^([0-5]{1}|[0-4]?\.[0-9]{0,2}|5\.[0]{0,2})$/;
                        $scope.HoldTimeOptions = [{ option: '0' }, { option: '1' }, { option: '2' }, { option: '3' }, { option: '4' }, { option: '5' }];
                        //#endregion ACSensorConfiguration preamble

                        //#region ACSensorConfiguration set functions
                        $scope.SetHoldTime = function (option) {
                            $scope.TempDevice.parameters[0].HoldTime = Number(option);
                        }

                        $scope.SelectFormula = function (option) {
                            $scope.TempDevice.parameters[0].FormulaIndex = option;
                        }

                        $scope.Recalculate = function (rangedLow, rangedHigh, formulaIndex) {
                            var formulaCalcRequest = {
                                parmeterIndex: 0,
                                rangedLow: rangedLow,
                                rangedHigh: rangedHigh,
                                formulaLow: -1,
                                formulaHigh: -1,
                                formulaIndex: formulaIndex
                            }

                            $http({
                                method: "put",
                                url: "/api/devices/formula/" + device.port,
                                data: formulaCalcRequest
                            }).success(function (formulaCalcReturn) {
                                //console.log("port0.ACSensorConfiguration.Apply: Calculated formula value=" + JSON.stringify(formulaCalcReturn));
                                $scope.TempDevice.parameters[0].Formulas[formulaIndex].FormulaLowThreshold = formulaCalcReturn.formulaLow;
                                $scope.TempDevice.parameters[0].Formulas[formulaIndex].FormulaHighThreshold = formulaCalcReturn.formulaHigh;
                            });
                        }
                        //#endregion ACSensorConfiguration set functions

                        //#region ACSensorConfiguration apply
                        $scope.Apply = function () {
                            //console.log("port0.ACSensorConfiguration.Apply: Device to update=" + JSON.stringify($scope.TempDevice));
                            $scope.TempDevice.parameters[0].RangedLowThreshold = Number($scope.TempDevice.parameters[0].RangedLowThreshold).toFixed(2);
                            $scope.TempDevice.parameters[0].RangedHighThreshold = Number($scope.TempDevice.parameters[0].RangedHighThreshold).toFixed(2);

                            $http({
                                method: "put",
                                url: "/api/devices/" + device.port + "/0",// Default to parameter 0 
                                data: $scope.TempDevice
                            }).success(function (updatedDevice) {
                                //console.log("port0.ACSensorConfiguration.Apply: Updated Device=" + JSON.stringify(updatedDevice));
                                $scope.Device.DeviceModel = updatedDevice;
                                $modalInstance.close();
                            });
                        };
                        //#endregion ACSensorConfiguration apply

                        //#region ACSensorConfiguration cancel
                        $scope.Cancel = function () {
                            $modalInstance.dismiss('cancel');
                        };
                        //#endregion ACSensorConfiguration cancel
                    }
                });
            };
            //#endregion Configure device            

            //#region Socket messages
            socketManager.on('ACSensor', function (message) {
                if (message.Port == $scope.ThisPort) {
                    //console.log("Port" + $scope.ThisPort + ".Controller.SocketOn: ACSensor=" + JSON.stringify(message));

                    switch (message.Message) {
                        case "Value":
                            switch (message.Parameter.Id) {
                                case "AC":
                                    $scope.Device.DeviceModel.parameters[0] = message.Parameter;
                                    break;
                            }
                            break;

                        case "EventLog":
                            $scope.EventLog = message.Parameter;
                            break;
                    }
                }
            });
            //#endregion Socket messages
            //#endregion ACSensor device

        }
        //#endregion Controller: port3
    ])
;