﻿// Requires for the GPS Interface
var serialport = require('serialport');
var nmea = require('nmea');
var gpsPort;

//---------------------------------------------
// Initializing Logging infrastructure
//#region Logging Infrastructure
// Start the logger
var log4js = require('log4js');

//config
log4js.configure({
    appenders: [
        { type: 'console' },
    ]
});
var logger = log4js.getLogger("test");
logger.setLevel(log4js.levels.ALL);// Toggle to this if you want all log info
//#endregion Logging Infrastructure


// Process the GPS interface
logger.info("GPS Interface has been initialized.")
gpsPort = new serialport.SerialPort('/dev/ttyO4', {
    baudrate: 9600,
    parser: serialport.parsers.readline('\r\n')
});
var oldLocation = { latitude: 0, longitude: 0 };

gpsPort.on('data', function (line) {
    try {
        var gpsData = nmea.parse(line);
        logger.info("Sentence=" + gpsData.sentence + " line=" + line);
        if (gpsData.sentence === 'GGA') {
            
            var latDecimal = gpsData.lat.indexOf(".");
            var latBase = gpsData.lat.substring(0, latDecimal - 2);
            var latMinutes = gpsData.lat.substring(latDecimal - 2, gpsData.lat.length)
            var latitude = (parseFloat(latBase) + parseFloat(latMinutes) / 60).toFixed(4);
            logger.info("latitude raw=" + gpsData.lat + " Base=" + latBase + " Minutes=" + latMinutes + " Latitude=" + latitude);
            
            var lonDecimal = gpsData.lon.indexOf(".");
            var lonBase = gpsData.lon.substring(0, lonDecimal - 2);
            var lonMinutes = gpsData.lon.substring(lonDecimal - 2, gpsData.lon.length)
            var longitude = (parseFloat(lonBase) + parseFloat(lonMinutes) / 60).toFixed(4);
            logger.info("longitude raw=" + gpsData.lon + " Base=" + lonBase + " Minutes=" + lonMinutes + " Longitude=" + longitude);
            
            var peakLocation = {
                latitude : latitude,
                longitude : -longitude
            }
            
            //io.sockets.emit("PeakLocation", peakLocation);
            if (peakLocation.latitude != oldLocation.latitude || peakLocation.longitude != oldLocation.longitude) {
                logger.info('GPS Interface - GGA data.lat=' + peakLocation.latitude + ' data.lon=' + peakLocation.longitude);
            }
            oldLocation = peakLocation;
        }
    }
    catch (error) {
        logger.error('GPS Interface - NMEA parsing error=' + JSON.stringify(error));
        
    }
});