﻿// Dependencies and Globals
//#region Dependencies and Globals
var rc4a = require('rc4a');
var moment = require('moment');
var fs = require('fs');
var cp = require('child_process');

var PeakPassPhrase = "TotallyAwesomeSuperCool!";// This is our super duper secure phrase
var PeakUser = {};
var PeakLoginAttempts = [];
var SUMMIT_LOGIN_ATTEMPTS = 10;

var Logger;// Logging global

//#endregion Dependencies and Globals

// Initialize
//#region Initialize
exports.Initialize = function (logger) {
    Logger = logger;// Globalize
    
    try {
        // Try to read the users file ...
        PeakUser = require("./users.json");
    }
    catch (error) {
        // Something went wrong, so get golden copy from ./default directory
        //  This will also happen on a fresh install of crest because users.json doesn't exist,
        //  and after a factory reset
        Logger.error("Users.LoginUsers: Unable to find user accounts, loading factory default user accounts. Error=" + error);
        cp.exec("cp ./factory/users.json ./users.json", function (error, stdout, stderr) {
            Logger.trace("Users.LoginUsers: ./factory/users.json ./users.json - success=" + stdout);
            if (error !== null) {
                Logger.error("Users.LoginUsers: Error in recovering factory default user accounts. Error=" + error);
            } else {
                // Sync the results so the files are made persistent
                cp.exec("sync", function (error, stdout, stderr) {
                    if (error !== null) {
                        Logger.error("Users.LoginUsers: Error in sync after loading factory default user accounts. Error=" + error);
                    } else {
                        PeakUser = require("./users.json");
                        Logger.info("Users.LoginUsers: Successfully recovered factory default user accounts.");
                    }
                });                
            }
        });
    }
}
//#endregion Initialize

// Get Peak User
//#region Get Peak User
exports.GetPeakUser = function (callback) {
    // For now, we'll always send decrypted password, so it's easier for client
    //  *** FUTURE: send encrypted password, because client will be able to manage RC4A
    var peakUser = JSON.parse(JSON.stringify(PeakUser));
    peakUser.password = rc4a.decrypt(PeakUser.password, PeakPassPhrase);
    callback(peakUser);
}
//#endregion Get Peak User

// Put Peak User
//#region Get Peak User
exports.PutPeakUser = function (peakUser, callback) {
    // For now, we'll always get decrypted password, so it's easier for client
    //  Therefore, we must encrypt before saving
    
    // Update PeakUser
    PeakUser.user = peakUser.user;
    PeakUser.password = rc4a.encrypt(peakUser.password, PeakPassPhrase);
    PeakUser.lastlogin = moment();
    peakUser.lastlogin = PeakUser.lastlogin;
    
    // Update users.json with new PeakUser settings to ensure persistence
    fs.writeFile('users.json', JSON.stringify(PeakUser), function (error) {
        if (error) {
            Logger.error("users.PutPeakUser: User="+peakUser.user+" account update write error. Error=" + error);
        }
        Logger.info("users.PutPeakUser: User="+peakUser.user+" account has been udpated");
        callback(peakUser);
    });
}
//#endregion Put Peak User

// GetPeakLoginAttempts
//#region GetPeakLoginAttempts
//  callback    - callback: PeakLoginAttempts
//#region Login Attempts
exports.GetPeakLoginAttempts = function (callback) {
    callback(PeakLoginAttempts);
}
//#endregion GetPeakLoginAttempts

// Login User
//  login       - login credentials
//  ip          - IP address of client
//  callback    - callback: success=true, failure=false
//#region Login User
exports.LoginUser = function (login, ip, callback) {
    //Logger.info("Users.LoginUsers: Entered=" + JSON.stringify(login) + " Actual=" + JSON.stringify(PeakUser));
    //Logger.info("Users.LoginUsers: Entered password=" + login.password + " Actual password=" + rc4a.decrypt(PeakUser.password, PeakPassPhrase));
    
    var success = false;
    var loginTime = moment();
    if ((login.user === PeakUser.user) && (login.password === rc4a.decrypt(PeakUser.password,PeakPassPhrase))) {
        
        // Login was successful, save the login time
        PeakUser.lastlogin = loginTime;
        
        // Save the users file with new login time
        // Update ports.json to ensure persistence
        fs.writeFile('users.json', JSON.stringify(PeakUser), function (error) {
            if (error) {
                Logger.error("Users.LoginUsers: Error writing users account after successful login. Error=" + error);
                success = false;
            } else {
                Logger.info("Users.LoginUsers: Successful login - " + PeakUser.user + " (from:" + ip + ") has successfully logged in at " + PeakUser.lastlogin);
                success = true;
            }
            PeakLoginAttempts.unshift({ "time": loginTime, "status": "Successful Login", "user": login.user, "ip": ip });
            if (PeakLoginAttempts.length > SUMMIT_LOGIN_ATTEMPTS) PeakLoginAttempts.splice(-1);
            callback(success);
        });
    }
    else {
        // Credentials do not match ...
        Logger.error("Users.LoginUsers: Failed login by" + PeakUser.user + " (from:" + ip + ") at " + loginTime);
        PeakLoginAttempts.unshift({ "time": loginTime, "status": "Credentials did not match", "user": login.user, "ip": ip });
        if (PeakLoginAttempts.length > SUMMIT_LOGIN_ATTEMPTS) PeakLoginAttempts.splice(-1);
        callback(success);
    }
}
//#endregion Login User